# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com


'''
@author: clarkmatthew
extension of the boto instance class, with added convenience methods + objects
Add common instance test routines to this class

Examples:
from eucaops import Eucaops
from eutester.windows_instance import WinInstance
tester = Eucaops(credpath='eucarc-10.111.5.80-eucalyptus-admin')
wins = WinInstance.make_euinstance_from_instance(tester.get_instances(idstring='i-89E13DA8')[0], tester=tester, keypair='test')
vol = tester.get_volume(status='available', zone=wins.placement)
wins.attach_volume(vol)



'''
from boto.ec2.instance import Instance
from eutester import Eutester
#from eucaops import Eucaops
from eutester.euvolume import EuVolume
from eutester import eulogger
from eutester.taggedresource import TaggedResource
from random import randint
from datetime import datetime
import winrm_connection
import socket
import sys
import os
import re
import time
import copy
import types
import operator


class WinInstanceDiskType():
    gigabyte = 1073741824
    megabyte = 1048576
    def __init__(self, win_instance, wmic_dict):
        self.check_dict_requires(wmic_dict)
        self.__dict__ =  self.convert_numbers_in_dict(copy.copy(wmic_dict))
        self.win_instance = win_instance
        self.size_in_gb = self.get_size_in_gb()
        self.size_in_mb = self.get_size_in_mb()
        self.size = long(self.size or 0)
        self.last_updated = time.time()
        self.setup()

    def setup(self):
        raise Exception('Not Implemented')

    def check_dict_requires(self, wmic_dict):
        raise Exception('Not Implemented')

    def convert_numbers_in_dict(self, dict):
        #convert strings representing numbers to ints
        for key in dict:
            value = str(dict[key])
            if (re.search("\S", str(dict[key])) and not re.search("\D", str(dict[key]))):
                dict[key] = long(dict[key])
        return dict

    def get_partition_ids(self):
        retlist = []
        for part in self.disk_partitions:
            retlist.append(part.deviceid)
        return retlist

    def get_logicaldisk_ids(self):
        retlist = []
        for part in self.disk_partitions:
            retlist.extend(part.get_logicaldisk_ids())
        return retlist

    def get_size_in_gb(self):
        '''
        Attempts to convert self.size from bytes to gigabytes as well as round up > .99 to account for a differences
        in how the size is represented
        '''
        self.size = int(self.size or 0)
        gigs = self.size / self.gigabyte
        if (self.size % self.gigabyte) /float(self.gigabyte) > .99:
            gigs += 1
        return gigs

    def get_size_in_mb(self):
        '''
        Attempts to convert self.size from bytes to gigabytes as well as round up > .99 to account for a differences
        in how the size is represented
        '''
        self.size = int(self.size or 0)
        mb = self.size / self.megabyte
        if (self.size % self.megabyte) /float(self.megabyte) > .99:
            mb += 1
        return mb

    def print_self(self):
        self.get_summary(printmethod=self.win_instance.debug)

    def get_summary(self, printheader=True, printmethod=None):
        raise Exception('Method not implemented')

    def get_line(self, length):
        line = ""
        for x in xrange(0,int(length)):
            line += "-"
        return "\n" + line + "\n"

    def print_self_full(self, printmethod=None):
        '''
        formats and prints self.dict
        '''
        self.win_instance.print_dict(dict=self.__dict__, printmethod=printmethod)



class WinInstanceDiskDrive(WinInstanceDiskType):

    def setup(self):
        if not hasattr(self, 'serialnumber'):
            self.serialnumber = ''
        if not hasattr(self, 'caption'):
            self.caption = ''
            if hasattr(self, 'model'):
                self.caption = self.model
            else:
                self.model = self.caption
        self.cygwin_scsi_drive = self.win_instance.get_cygwin_scsi_dev_for_windows_drive(windisk=self)
        self.update_ebs_info()
        self.disk_partitions = []

    def check_dict_requires(self, wmic_dict):
        if not ('deviceid' in wmic_dict and
                'size' in wmic_dict and
                ('caption' in wmic_dict  or 'model in wmic_dict') and
                'index' in wmic_dict):
            raise Exception('wmic_dict passed does not contain needed attributes; deviceid, size, caption, and index')


    def get_partition_ids(self):
        retlist = []
        for part in self.disk_partitions:
            retlist.append(part.deviceid)
        return retlist

    def get_logicaldisk_ids(self):
        retlist = []
        for part in self.disk_partitions:
            retlist.extend(part.get_logicaldisk_ids())
        return retlist

    def update_md5_info_from_ebs(self):
        self.md5 = None
        self.md5len = None
        for vol in self.win_instance.attached_vols:
            if vol.guestdev == self.deviceid:
                if not vol.md5:
                    vol.md5len = 1024
                    vol.md5 = self.win_instance.get_dev_md5(self.cygwin_scsi_drive, vol.md5len)
                self.md5 = vol.md5
                self.md5len = vol.md5len
                break

    def update_ebs_info_from_serial_number(self):
        '''
        Attempts to parse the serial number field from an EBS volume and find the correlating ebs volume
        example format: vol-81C13EA4-dev-sdg
        '''
        if re.match("^vol-", self.serialnumber):
            split = self.serialnumber.split('-')
            self.ebs_volume = str(split[0]) + "-" + str(split[1])
            self.ebs_cloud_dev = "/" + str(split[2]) + "/" + str(split[3])
        else:
            self.ebs_volume = ''
            self.ebs_cloud_dev = ''


    def update_ebs_info(self):
        self.update_ebs_info_from_serial_number()
        if not self.ebs_volume:
            if self.index == 0 and self.win_instance.root_device_type == 'ebs':
                bdm = self.win_instance.block_device_mapping[self.win_instance.root_device_name]
                self.ebs_volume = bdm.volume_id
            else:
                for vol in self.win_instance.attached_vols:
                    if vol.guestdev == self.deviceid:
                        self.ebs_volume = vol.id
                        break
        if not self.ebs_cloud_dev and self.ebs_volume:
            volume = self.win_instance.tester.get_volume(volume_id=self.ebs_volume)
            if hasattr(volume,'attach_data') and volume.attach_data:
                self.ebs_cloud_dev = volume.attach_data.device
        self.update_md5_info_from_ebs()




    def get_summary(self, printheader=True, printmethod=None):
        buf = ""
        deviceid = 20
        size = 16
        sizegb = 7
        ebsvol = 12
        serialnumber = 24
        caption = 36
        part_count = 6
        logical_ids = 8
        cygdrive = 10
        md5 = 32
        header = "DISKDRIVE DEV ID".center(deviceid) + "|" + \
                 "SIZE B".center(size) + "|" + \
                 "SIZE GB".center(sizegb) + "|" + \
                 "EBS VOL".center(ebsvol) + "|" + \
                 "CAPTION".center(caption) + "|" + \
                 "PARTS".center(part_count) + "|" + \
                 "LOGICAL".center(logical_ids) + "|" + \
                 "CYGDRIVE".center(cygdrive) + "|" + \
                 "SERIAL NUMBER".center(serialnumber) + "|" + \
                 "MD5 CHECK SUM".center(md5) + "|"

        summary = str(self.deviceid).center(deviceid) + "|" + \
                  str(self.size).center(size) + "|" + \
                  str(self.size_in_gb).center(sizegb) + "|" + \
                  str(self.ebs_volume).center(ebsvol) + "|" + \
                  str(self.caption).center(caption) + "|" + \
                  str(self.partitions).center(part_count) + "|" + \
                  str(",".join(str(x) for x in self.get_logicaldisk_ids())).center(logical_ids) + "|" + \
                  str(self.cygwin_scsi_drive).center(cygdrive) + "|" + \
                  str(self.serialnumber).center(serialnumber) + "|" + \
                  str(self.md5).center(md5) + "|"

        length = len(header)
        if len(summary) > length:
            length = len(summary)
        line = self.get_line(length)
        if printheader:
            buf += line + header + line
        buf += summary + line
        if printmethod:
            printmethod(buf)
        return buf


class WinInstanceDiskPartition(WinInstanceDiskType):

    def setup(self):
        #self.cygwin_scsi_drive = self.win_instance.get_cygwin_scsi_dev_for_windows_drive(drive_id=self.deviceid)
        self.logicaldisks = []
        #Set values in case 'brief' was used when fetching partitions
        if not hasattr(self,'deviceid'):
            self.deviceid = self.name
        if not hasattr(self,'bootable'):
            self.bootable = self.bootpartition
        if not hasattr(self,'diskindex'):
            self.diskindex = self.get_disk_index_from_name()

    def check_dict_requires(self, wmic_dict):
        if not ('name' in wmic_dict and
                'size' in wmic_dict and
                'bootpartition' in wmic_dict and
                'index' in wmic_dict):
            raise Exception('wmic_dict passed does not contain needed attributes; deviceid, size, index and bootable')


    def get_disk_index_from_name(self):
        diskindex = None
        diskindexstring = self.name.split(',')[0]
        if re.search('disk', diskindexstring, re.IGNORECASE):
            diskindex = int(diskindexstring.split('#')[1])
        return diskindex

    def get_logicaldisk_ids(self):
        retlist = []
        for disk in self.logicaldisks:
            retlist.append(disk.deviceid)
        return retlist

    def get_summary(self, printheader=True, printmethod=None):
        buf = ""
        deviceid = 24
        size = 16
        sizegb = 12
        sizemb = 12
        bootable = 10
        header = "PARTITION DEV ID".center(deviceid) + "|" + \
                 "SIZE B".center(size) + "|" + \
                 "SIZE GB".center(sizegb) + "|" + \
                 "SIZE MB".center(sizemb) + "|" + \
                 "BOOTABLE".center(bootable) + "|"

        summary = str(self.deviceid).center(deviceid) + "|" + \
                  str(self.size).center(size) + "|" + \
                  str(self.size_in_gb).center(sizegb) + "|" + \
                  str(self.size_in_mb).center(sizemb) + "|" + \
                  str(self.bootable).center(bootable) + "|"

        length = len(header)
        if len(summary) > length:
            length = len(summary)
        line = self.get_line(length)
        if printheader:
            buf += line + header + line
        buf += summary + line
        if printmethod:
            printmethod(buf)
        return buf


class WinInstanceLogicalDisk(WinInstanceDiskType):

    def setup(self):
        self.cygwin_scsi_drive = self.win_instance.get_cygwin_scsi_dev_for_windows_drive(windisk=self)
        self.partition = None

    def check_dict_requires(self, wmic_dict):
        if not ('deviceid' in wmic_dict and
                'size' in wmic_dict and
                'description' in wmic_dict and
                'freespace' in wmic_dict and
                'filesystem' in wmic_dict):
            raise Exception('wmic_dict passed does not contain needed attributes; deviceid, size, and description')

    def get_summary(self, printheader=True, printmethod=None):
        buf = ""
        deviceid = 24
        size = 16
        freespace = 16
        filesystem = 24
        description = 30
        cygdrive = 10
        header = "LOGICAL DEV ID".center(deviceid) + "|" + \
                 "SIZE".center(size) + "|" + \
                 "FREE SPACE".center(freespace) + "|" + \
                 "FILE SYSTEM".center(filesystem) + "|" + \
                 "DESCRIPTION".center(description) + "|" + \
                 "CYGDRIVE".center(cygdrive) + "|"
        summary = str(self.deviceid).center(deviceid) + "|" + \
                  str(self.size).center(size) + "|" + \
                  str(self.freespace).center(freespace) + "|" + \
                  str(self.filesystem).center(filesystem) + "|" + \
                  str(self.description).center(description) + "|" + \
                  str(self.cygwin_scsi_drive).center(cygdrive) + "|"
        length = len(header)
        if len(summary) > length:
            length = len(summary)
        line = self.get_line(length)
        if printheader:
            buf += line + header + line
        buf += summary + line
        if printmethod:
            printmethod(buf)
        return buf



class WinInstance(Instance, TaggedResource):
    gigabyte = 1073741824
    megabyte = 1048576

    @classmethod
    def make_euinstance_from_instance(cls,
                                      instance,
                                      tester,
                                      debugmethod = None,
                                      keypair=None,
                                      keypath=None,
                                      password=None,
                                      username="Administrator",
                                      auto_connect = True,
                                      verbose=True,
                                      timeout=120,
                                      private_addressing = False,
                                      reservation = None,
                                      cmdstart=None,
                                      try_non_root_exec=True,
                                      winrm_port='5985',
                                      winrm_protocol='http',
                                      rdp_port='3389',
                                      rootfs_device = "sda",
                                      block_device_prefix = "sd",
                                      bdm_root_vol = None,
                                      attached_vols = [],
                                      virtio_blk = True,
                                      cygwin_path = None,
                                      disk_update_interval=10,
                                      retry=2
                                      ):
        '''
        Primary constructor for this class. Note: to avoid an ssh session within this method, provide keys, username/pass later.
        Arguments:
        instance - mandatory- a Boto instance object used to build this euinstance object
        keypair - optional- a boto keypair object used for creating ssh connection to the instance
        username - optional- string used to create ssh connection as an alternative to keypair
        password - optional- string used to create ssh connection to this instance as an alternative to keypair
        exec_password -optional -string used for su or sudo where prompted for password, will default to 'password'
        auto_connect -optional -boolean, if True will attempt to automatically create an ssh session for this instance
        try_non_root_exec -optional -boolean, if True will attempt to use sudo if available else su -c to execute privileged commands
        timeout - optional- integer used for ssh connection timeout
        debugmethod - optional - method, used for debug output
        verbose - optional - boolean to determine if debug is to be printed using debug()
        retry - optional - integer, ssh connection attempts for non-authentication failures
        '''
        newins = WinInstance(instance.connection)
        newins.__dict__ = instance.__dict__
        newins.tester = tester
        newins.winrm_port = winrm_port
        newins.rdp_port = rdp_port
        newins.winrm_protocol = winrm_protocol
        newins.debugmethod = debugmethod
        if newins.debugmethod is None:
            newins.logger = eulogger.Eulogger(identifier= str(instance.id))
            newins.debugmethod= newins.logger.log.debug

        if (keypair is not None):
            if isinstance(keypair,types.StringTypes):
                keyname = keypair
                keypair = tester.get_keypair(keyname)
            else:
                keyname = keypair.name
            newins.keypath = keypath or os.getcwd() + "/" + keyname + ".pem"
        newins.keypair = keypair

        newins.password = password
        newins.username = username
        newins.verbose = verbose
        newins.attached_vols=attached_vols
        newins.timeout = timeout
        newins.virtio_blk  = virtio_blk
        newins.disk_update_interval = disk_update_interval
        newins.retry = retry
        newins.rootfs_device =  rootfs_device
        newins.block_device_prefix = block_device_prefix
        newins.private_addressing = private_addressing
        newins.reservation = reservation or newins.get_reservation()
        if newins.reservation:
            newins.security_groups = newins.tester.get_instance_security_groups(newins)
        else:
            newins.security_groups = None
        newins.laststate = newins.state
        newins.cmdstart = cmdstart
        newins.auto_connect = auto_connect
        newins.set_last_status()
        newins.update_vm_type_info()
        newins.cygwin_path = cygwin_path
        newins.system_info = None
        newins.diskdrives = []
        newins.disk_partitions = []
        newins.logicaldisks = []
        newins.cygwin_dev_map  = {}
        #newins.set_block_device_prefix()
        if newins.root_device_type == 'ebs':
            try:
                volume = newins.tester.get_volume(volume_id = newins.block_device_mapping.get(newins.root_device_name).volume_id)
                newins.bdm_root_vol = EuVolume.make_euvol_from_vol(volume, tester=newins.tester,cmdstart=newins.cmdstart)
            except:pass
        else:
            newins.bdm_root_vol = None
        newins.winrm = None
        if newins.auto_connect and newins.state == 'running':
            newins.connect_to_instance(timeout=timeout)
        return newins

    def update(self):
        super(WinInstance, self).update()
        self.set_last_status()

    def update_vm_type_info(self):
        self.vmtype_info =  self.tester.get_vm_type_from_zone(self.placement,self.instance_type)
        return self.vmtype_info


    def set_last_status(self,status=None):
        self.laststate = self.state
        self.laststatetime = time.time()
        self.age_at_state = self.tester.get_instance_time_launched(self)
        #Also record age from user's perspective, ie when they issued the run instance request (if this is available)
        if self.cmdstart:
            self.age_from_run_cmd = "{0:.2f}".format(time.time() - self.cmdstart)
        else:
            self.age_from_run_cmd = None

    def print_dict(self, dict=None, printmethod=None):
        '''
        formats and prints
        '''
        printmethod = printmethod or self.debug
        buf = "\n"
        dict = dict or self.__dict__
        longest_key = 0
        for key in dict:
            if len(key) > longest_key:
                longest_key = len(key)
        for key in dict:
            buf += str(key).ljust(longest_key) + " -----> :" + str(dict[key]) + "\n"
        printmethod(buf)

    def printself(self,title=True, footer=True, printmethod=None):
        instid = 11
        emi = 13
        resid = 11
        laststate =10
        privaddr = 10
        age = 13
        vmtype = 12
        rootvol = 13
        cluster = 25
        pubip = 16

        if self.bdm_root_vol:
            bdmvol = self.bdm_root_vol.id
        else:
            bdmvol = None
        reservation_id = None
        if self.reservation:
            reservation_id = self.reservation.id
        header = ""
        buf = ""
        if title:
            header = str('INST_ID').center(instid) +'|' + \
                     str('EMI').center(emi) + '|' +  \
                     str('RES_ID').center(resid) + '|' +  \
                     str('LASTSTATE').center(laststate) + '|' +  \
                     str('PRIV_ADDR').center(privaddr) + '|' +  \
                     str('AGE@STATUS').center(age) + '|' +  \
                     str('VMTYPE').center(vmtype) + '|' +  \
                     str('ROOT_VOL').center(rootvol) + '|' +  \
                     str('CLUSTER').center(cluster) + '|' +  \
                     str('PUB_IP').center(pubip) + '|' +  \
                     str('PRIV_IP')
        summary = str(self.id).center(instid) + '|' + \
                  str(self.image_id).center(emi) + '|' +  \
                  str(reservation_id).center(resid) + '|' +  \
                  str(self.laststate).center(laststate) + '|' +  \
                  str(self.private_addressing).center(privaddr) + '|' + \
                  str(self.age_at_state).center(age) + '|' +  \
                  str(self.instance_type).center(vmtype) + '|' +  \
                  str(bdmvol).center(rootvol) + '|' +  \
                  str(self.placement).center(cluster) + '|' + \
                  str(self.ip_address).center(pubip) + '|' + \
                  str(self.private_ip_address).rstrip()

        length = len(header)
        if len(summary) > length:
            length = len(summary)
        line = self.get_line(length)
        if title:
            buf = line + header + line
        buf += summary
        if footer:
            buf += line
        if printmethod:
            printmethod(buf)
        return buf


    def get_line(self, length):
        line = ""
        for x in xrange(0,int(length)):
            line += "-"
        return "\n" + line + "\n"

    def get_password(self, private_key_path=None, key=None, dir=None, exten=".pem", encoded=True):
        '''
        :param private_key_path: private key file used to decrypt password
        :param key: name of private key
        :param dir: Path to private key
        :param exten: extension of private key
        :param encoded: boolean of whether string returned from server is Base64 encoded
        :return: decrypted password
        '''
        if self.password is None:
            self.password = self.tester.get_windows_instance_password(self,
                                                                      private_key_path=private_key_path,
                                                                      key=key,
                                                                      dir=dir,
                                                                      exten=exten,
                                                                      encoded=encoded)
        return self.password


    def reset_ssh_connection(self, timeout=None):
        # todo: Remove ssh reference from this method, use something like reset_instance_connection, etc..
        self.debug('Note ssh not implemented at this time, using winrm for shell access instead...')
        return self.reset_winrm_connection(timeout=timeout)

    def reset_winrm_connection(self, timeout=None, force=False):
        # todo:
        timeout = timeout or self.timeout
        self.debug('reset_winrm_connection for:'+str(self.id))
        if self.password is None:
            self.get_password()
        if self.username is None or self.password is None:
            #Allow but warn here as this may be a valid negative test
            self.debug('Warning username and/or password were None in winrm connnection?')
        #Create a new winrm interface if this is a new instance or an attribute has changed...
        try:
            #Check the port in order to provide debug if the connection fails
            self.test_port_status(port=self.winrm_port, ip=self.ip_address)
        except:pass
        if force or not (self.winrm and \
                         self.winrm.hostname == self.ip_address and \
                         self.winrm.username == self.username and \
                         self.winrm.password == self.password):
            if self.winrm:
                self.winrm.close_shell()
            self.winrm = winrm_connection.Winrm_Connection(hostname = self.ip_address,
                                                           username = self.username,
                                                           password = self.password,
                                                           port = self.winrm_port,
                                                           protocol = self.winrm_protocol,
                                                           debug_method = self.debug,
                                                           verbose=True
                                                           )


    def get_reservation(self):
        res = None
        try:
            res = self.tester.get_reservation_for_instance(self)
        except Exception, e:
            self.update()
            self.debug('Could not get reservation for instance in state:' + str(self.state) + ", err:" + str(e))
        return res


    def connect_to_instance(self, wait_for_boot=180, timeout=120):
        '''
        Attempts to connect to an instance via ssh.
        wait_for_boot=time to wait, allowing guest to boot before attempting to poll for ports active status
        timeout - optional - time in seconds to wait when polling port(s) status(s) before failure

        '''
        try:
            self.poll_for_port_status_with_boot_delay(waitforboot=wait_for_boot, timeout=timeout)
        except Exception, e:
            self.debug('Warning failed to poll port status:' + str(e))
        self.debug("Attempting to create connection to instance:" + self.id)
        attempts = 0
        start = time.time()
        elapsed = 0
        if self.winrm is not None:
            self.winrm.close_shell()
        self.winrm = None
        while (elapsed < timeout):
            attempts += 1
            try:
                self.update()
                self.reset_winrm_connection()
                self.debug('Try some sys...')
                self.sys("whoami")
            except Exception, se:
                tb = self.tester.get_traceback()
                self.debug('Caught exception attempting to connect winrm shell:\n'+ str(tb) + str(se))
                elapsed = int(time.time()-start)
                self.debug('connect_to_instance: Attempts:'+str(attempts)+', elapsed:'+str(elapsed)+'/'+str(timeout))
                if self.winrm is not None:
                    self.winrm.close_shell()
                    self.winrm = None
                time.sleep(5)
                pass
            else:
                break
        elapsed = int(time.time()-start)
        if self.winrm is None:
            raise Exception(str(self.id)+":Failed establishing management connection to instance, elapsed:"+str(elapsed)+
                            "/"+str(timeout))
        self.update_system_and_disk_info()
        self.init_attached_volumes()

    def update_root_device_diskdrive(self):
        if not self.root_device_type == 'ebs':
            return
        for disk in self.diskdrives:
            if disk.index == 0:
                if disk.ebs_volume:
                    for vol in self.attached_vols:
                        if vol.id == disk.ebs_volume:
                            if not disk.md5:
                                disk.update_md5_info_from_ebs()
                            return
                    volume = self.tester.get_volume(volume_id=disk.ebs_volume)
                    if not isinstance(volume, EuVolume):
                        volume = EuVolume.make_euvol_from_vol(volume, self.tester)
                    volume.guestdev = disk.deviceid
                    volume.md5len = 1024
                    volume.md5 = self.get_dev_md5(disk.cygwin_scsi_drive, volume.md5len)
                    self.attached_vols.append(volume)
                    disk.update_md5_info_from_ebs()
                    return

    def get_volume_from_attached_list_by_id(self, volume_id):
        for vol in self.attached_vols:
            if vol.id == volume_id:
                return vol


    def update_system_and_disk_info(self):
        try:
            self.update_system_info()
        except Exception, sie:
            tb = self.tester.get_traceback()
            self.debug(str(tb) + "\nError updating system info:" + str(sie))
        try:
            self.update_disk_info()
            self.update_root_device_diskdrive()
            self.print_partition_summary()
            self.print_logicaldisk_summary()
            self.print_diskdrive_summary()
        except Exception, ude:
            tb = self.tester.get_traceback()
            self.debug(str(tb) + "\nError updating disk info:" + str(ude))


    def has_sudo(self):
        return False


    def debug(self,msg,traceback=1,method=None,frame=False):
        '''
        Used to print debug, defaults to print() but over ridden by self.debugmethod if not None
        msg - mandatory -string, message to be printed
        '''
        if ( self.verbose is True ):
            self.debugmethod(msg)

    def sys(self, cmd, verbose=True, code=None, include_stderr=False, enable_debug=False, timeout=None):
        '''
        Issues a command against the ssh connection to this instance
        Returns a list of the lines from stdout+stderr as a result of the command
        cmd - mandatory - string, the command to be executed
        verbose - optional - boolean flag to enable debug
        timeout - optional - command timeout in seconds
        '''
        if (self.winrm is None):
            raise Exception("WinInstance winrm connection is None")
        return self.winrm.sys(command=cmd, include_stderr=include_stderr, timeout=timeout, verbose=verbose, code=code)




    def test_rdp_port_status(self, ip=None, port=3389, timeout=10):
        '''
        Description: Attempts to test that the host is accepting tcp connections to the RDP port
        '''
        ip = ip or self.ip_address
        return self.test_port_status(ip=ip, port=port, timeout=timeout)


    def test_port_status(self, port, ip=None, timeout=5, tcp=True, verbose=True):
        ip = ip or self.ip_address
        return self.tester.test_port_status(ip, int(port), timeout=timeout, tcp=tcp, verbose=verbose)

    def poll_for_port_status_with_boot_delay(self, interval=15, ports=[], socktimeout=5,timeout=180, waitforboot=300):
        '''
        Make sure some time has passed before we test on the guest side before running guest test...

        '''
        launch_seconds = self.tester.get_instance_time_launched(self)
        sleeptime =  0 if launch_seconds > waitforboot else (waitforboot - launch_seconds)
        self.debug("Instance was launched "+str(launch_seconds)+" seconds ago, waiting:"+str(sleeptime)+" for instance to boot")
        time.sleep(sleeptime)
        return self.poll_for_ports_status(ports,
                                          ip=self.ip_address,
                                          interval=interval,
                                          socktimeout=socktimeout,
                                          timeout=timeout)

    def wait_for_time_since_launch(self,waitforboot=420):
        '''
        When using larger instance store images, this can allow for the delays caused by image size/transfer.
        '''
        boot_seconds = self.tester.get_instance_time_launched(self)
        sleeptime =  0 if boot_seconds > waitforboot else (waitforboot - boot_seconds)
        self.debug("Instance was launched "+str(boot_seconds)+"/"+str(waitforboot) + " seconds ago, waiting:"+str(sleeptime)+" for instance to boot")
        start = time.time()
        elapsed = 0
        print "Waiting for Windows to fully boot:",
        while elapsed < sleeptime:
            print "Waiting for Windows to fully boot:"+str(sleeptime-elapsed),
            time.sleep(5)
            elapsed=int(time.time()-start)
        self.debug("test_wait_for_instance_boot: done waiting, instance up for "+str(waitforboot)+" seconds")

    def poll_for_ports_status(self, ports=[], ip=None, interval=10, socktimeout=5, timeout=180):
        ip = ip or self.ip_address
        ports = ports or [self.rdp_port, self.winrm_port]
        start = time.time()
        elapsed = 0
        attempt = 0
        while elapsed < timeout:
            attempt +=1
            self.debug('test_poll_for_ports_status, ports: ' + ",".join(str(x) for x in ports) + ", attempt:"  + str(attempt))
            for port in ports:
                if elapsed < timeout:
                    try:
                        self.debug('Trying ip:port:' + str(self.ip_address) + ':' + str(port) + ", elapsed:" + str(elapsed))
                        self.test_port_status(ip=ip, port=int(port), timeout=5)
                        return
                    except socket.error, se:
                        self.debug('test_ports_status failed socket error:'+str(se[0]))
                        #handle specific errors here, for now just for debug...
                        ecode=se[0]
                        if ecode == socket.errno.ETIMEDOUT or ecode == "timed out":
                            self.debug("test_poll_for_ports_status: Connect "+str(ip)+":" +str(port)+ " timed out retrying. Time remaining("+str(timeout-elapsed)+")")
                    except Exception, e:
                        tb = self.tester.get_traceback()
                        self.debug(tb)
                        self.debug('test_poll_for_ports_status:'+str(ip)+':'+str(port)+' FAILED after attempts:'+str(attempt)+', elapsed:'+str(elapsed)+', err:'+str(e) )
            elapsed = int(time.time() -start)
            if elapsed < timeout:
                time.sleep(interval)

        raise Exception('test_poll_for_ports_status:'+str(ip)+':'+str(port)+' FAILED after attempts:'+str(attempt)+', elapsed:'+str(elapsed)+' seconds')

    def init_attached_volumes(self):
        syncdict = self.sync_attached_volumes_with_clouds_view()
        if syncdict['errors']:
            errmsg = 'Errors syncing guest volumes with cloud at init:' + ",".join(str(e) for e in syncdict['errors'])
            errmsg += 'Failed to sync guest volumes with cloud at init:' + ",".join(str(x) for x in syncdict['badvols'])
            raise Exception(errmsg)

    def sync_attached_volumes_with_clouds_view(self):
        badvols = []
        errors = []
        ret = {'errors':errors, 'badvols':badvols}
        #Get a list of volumes that the cloud believes are currently attached
        cloud_volumes = self.tester.get_volumes(attached_instance=self.id)
        #Make a copy of a list of volumes this instance thinks are currenlty attached
        locallist = copy.copy(self.attached_vols)

        for vol in cloud_volumes:
            for local_vol in locallist:
                if local_vol.id == vol.id:
                    locallist.remove(local_vol)
            if not isinstance(vol, EuVolume):
                vol = EuVolume.make_euvol_from_vol(vol, self.tester)
            try:
                self.update_volume_guest_info(volume=vol)
            except Exception, e:
                badvols.append(vol)
                errors.append(vol.id + ' Error syncing with cloud:' + str (e) + '. \n')
        for local_vol in locallist:
            badvols.append(local_vol)
            errors.append(local_vol.id + ' Error unattached volume found in guests attach list. \n')
        return ret



    def update_system_info(self):
        '''
        Gather basic system info for this windows instance object and store in self.system_info
        Example:
        # print wins.system_info.OS_NAME
          'Microsoft Windows 7 Professional'
        '''
        currentkey = None
        swap = re.compile('([!@#$%^&*. ])')
        info = self.sys('systeminfo')
        if self.system_info:
            system_info = self.system_info
        else:
            system_info = type('obj', (object,),{})
        if info:
            for line in info:
                if re.match("^\w.+:", line):
                    linevals = line.split(':')
                    currentkey = linevals.pop(0)
                    #clean up the key string...
                    currentkey = re.sub('[()]', '', currentkey)
                    currentkey = re.sub(swap, '_', currentkey)
                    currentkey = currentkey.lower()
                    value = ":".join(str(x) for x in linevals) or ""
                    setattr(system_info, currentkey, str(value).strip())
                elif currentkey:
                    #this is an additional value to our previous key
                    prev_value = getattr(system_info, currentkey)
                    if not isinstance(prev_value, types.ListType):
                        updated_value = [prev_value]
                    updated_value.append(str(line).strip())
                    setattr(system_info, currentkey, updated_value)
        self.system_info = system_info

    def get_cygwin_path(self, prefix="c:\\"):
        if self.cygwin_path:
            return self.cygwin_path
        path = None
        self.debug('Trying to find cygwin path...')
        out = self.sys('dir ' + str(prefix) + ' /B')
        for line in out:
            if re.search('cygwin', line):
                path = str(prefix) + str(line.strip()) + "\\"
                self.cygwin_path = path
                break
        return path

    def cygwin_curl(self, url):
        cygpath = self.get_cygwin_path()
        if cygpath is None:
            raise Exception('Could not find cygwin path on guest for curl?')
        curl = cygpath + 'bin\curl.exe '
        return self.sys(curl + str(url), code=0)




    def get_metadata(self, element_path='', prefix='latest/meta-data/', use_cygwin=True):
        """Return the lines of metadata from the element path provided"""
        ### If i can reach the metadata service ip use it to get metadata otherwise try the clc directly
        try:
            self.sys("ping -c 1 169.254.169.254", code=0, verbose=False)
            if use_cygwin:
                return self.cygwin_curl("http://169.254.169.254/"+str(prefix)+str(element_path))
            else:
                return self.sys("curl http://169.254.169.254/"+str(prefix)+str(element_path), code=0)
        except:
            if use_cygwin:
                return self.cygwin_curl("http://" + self.tester.get_ec2_ip()  + ":8773/"+str(prefix) + str(element_path))
            else:
                return self.sys("curl http://" + self.tester.get_ec2_ip()  + ":8773/"+str(prefix) + str(element_path), code=0)


    def print_diskdrive_summary(self,printmethod=None):
        printmethod = printmethod or self.debug
        if not self.diskdrives:
            printmethod('No disk drives to print?')
            return
        disklist = copy.copy(self.diskdrives)
        buf = (disklist.pop()).get_summary()
        for disk in disklist:
            buf += disk.get_summary(printheader=False)
        printmethod(buf)

    def print_partition_summary(self,printmethod=None):
        printmethod = printmethod or self.debug
        if not self.disk_partitions:
            printmethod('No disk partitions to print?')
            return
        partlist = copy.copy(self.disk_partitions)
        buf = (partlist.pop()).get_summary()
        for part in partlist:
            buf += part.get_summary(printheader=False)
        printmethod(buf)

    def print_logicaldisk_summary(self,printmethod=None):
        printmethod = printmethod or self.debug
        if not self.logicaldisks:
            printmethod('No disk disk_partitions to print?')
            return
        disklist = copy.copy(self.logicaldisks)
        buf = (disklist.pop()).get_summary()
        for disk in disklist:
            buf += disk.get_summary(printheader=False)
        printmethod(buf)


    def update_disk_info(self , forceupdate=False):
        if self.diskdrives:
            if not forceupdate and (time.time() - self.diskdrives[0].last_updated) <= self.disk_update_interval:
                return
        self.debug('Fetching updated disk info...')
        self.diskdrives = []
        self.disk_partitions = []
        self.logicaldisks = []
        self.diskdrives =  self.get_updated_diskdrive_info()
        self.disk_partitions = self.get_updated_partition_info()
        self.logicaldisks = self.get_updated_logicaldisk_info()
        self.associate_diskdrives_to_partitions()
        self.associate_partitions_to_logicaldrives()

    def get_updated_diskdrive_info(self):
        '''
        Populate self.diskdrives with WinInstanceDisk objects containing info parsed from wmic command.
        Since wmic doesn't seem to use delimeters this method attempts to derive the lengh of each column/header
        in order to parse out the info per disk.
        :pararm force: boolean. Will force an update, otherwise this method will wait a minimum of
        self.disk_update_interval before updating again.
        '''
        #cmd = "wmic diskdrive get  /format:textvaluelist.xsl"
        self.debug('Getting updated diskdrive info...')
        cmd = "wmic diskdrive list full"

        diskdrives = []
        for disk_dict in self.get_parsed_wmic_command_output(cmd):
            try:
                diskdrives.append(WinInstanceDiskDrive(self,disk_dict))
            except Exception, e:
                tb = self.tester.get_traceback()
                self.debug('Error attempting to create WinInstanceDiskDrive from following dict:')
                self.print_dict(dict=disk_dict)
                raise Exception(str(tb) + "\n Error attempting to create WinInstanceDiskDrive:" + str(e))
        self.debug('get_updated_diskdrive_info, Done')
        return diskdrives


    def get_updated_partition_info(self):
        '''
        Populate self.diskdrives with WinInstanceDisk objects containing info parsed from wmic command.
        Since wmic doesn't seem to use delimeters this method attempts to derive the lengh of each column/header
        in order to parse out the info per disk.
        :pararm force: boolean. Will force an update, otherwise this method will wait a minimum of
        self.disk_update_interval before updating again.
        '''
        self.debug('Getting udpated partition info...')
        cmd = "wmic partition list brief /format:textvaluelist.xsl"

        disk_partitions = []
        for part_dict in self.get_parsed_wmic_command_output(cmd):
            try:
                disk_partitions.append(WinInstanceDiskPartition(self,part_dict))
            except Exception, e:
                tb = self.tester.get_traceback()
                self.debug('Error attempting to create WinInstanceDiskPartition from following dict:')
                self.print_dict(dict=part_dict)
                raise Exception(str(tb) + "\n Error attempting to create WinInstanceDiskPartition:" + str(e))
        self.debug('get_updated_partition_info, Done')
        return disk_partitions


    def get_updated_logicaldisk_info(self):
        self.debug('Getting updated logicaldisk info...')
        cmd ='wmic logicaldisk list /format:textvaluelist.xsl'
        logicaldisks = []
        for part_dict in self.get_parsed_wmic_command_output(cmd):
            try:
                logicaldisks.append(WinInstanceLogicalDisk(self,part_dict))
            except Exception, e:
                tb = self.tester.get_traceback()
                self.debug('Error attempting to create WinInstanceLogicalDisk from following dict:')
                self.print_dict(dict=part_dict)
                raise Exception(str(tb) + "\n Error attempting to create WinInstanceLogicalDisk:" + str(e))
        self.debug('get_updated_logicaldisk_info, Done')
        return logicaldisks


    def associate_diskdrives_to_partitions(self):
        for disk in self.diskdrives:
            disk.disk_partitions = []
            for part in self.disk_partitions:
                if part.diskindex == disk.index:
                    disk.disk_partitions.append(part)

    def associate_partitions_to_logicaldrives(self, verbose=False):
        for part in self.disk_partitions:
            drive_id = None
            part.logicaldisks = []
            cmd = 'wmic partition where (DeviceID="Disk #' + str(part.diskindex) + \
                  ', Partition #' + str(part.index) + '") assoc /assocclass:Win32_LogicalDiskToPartition'
            output = self.sys(cmd, verbose=verbose, code=0)
            for line in output:
                if re.search('Win32_LogicalDisk.DeviceID',line):
                    try:
                        drive_id =  str(line.split()[0].split('=')[1]).replace('"','').strip()
                    except Exception, e:
                        tb = self.tester.get_traceback()
                        self.debug(str(tb)+ "\nError getting logical drive info:" + str(e))
                    if drive_id:
                        for disk in self.logicaldisks:
                            if re.match(disk.deviceid, drive_id):
                                part.logicaldisks.append(disk)
                                disk.partition = part
                                break

    def get_cygwin_scsi_dev_for_windows_drive(self, windisk=None, drive_id=""):
        '''
        param windisk: WinInstanceDiskType object. windisk.deviceid is used to look up the associated cygwin device
        param drive_id: String representing the deviceid. Can be used instead of passing a WinInstanceDiskType
        '''
        windisk_classname = ""
        update = False
        retries = 2
        if windisk:
            drive_id = windisk.deviceid
            windisk_classname = str(windisk.__class__).split('.').pop()
            #If this is a disk drive allow a retry which set the force update flag, otherwise don't force and retry
            if isinstance(windisk,WinInstanceDiskDrive):
                update = True
        if not drive_id:
            raise Exception('WinInstanceDiskType or string w/ device id not provided')

        self.debug('Attempting to get cygwin dev for windows drive:' + str(drive_id))
        self.update_cygwin_windows_device_map()
        for retry in xrange(0, retries):
            for device in self.cygwin_dev_map:
                if re.search("dev", device):
                    win_dev = str(self.cygwin_dev_map[device].split('\\').pop()).strip().upper()
                    formated_drive_id = str(drive_id.split('\\').pop()).strip().upper()
                    #self.debug('Attempt to match:"' + str(win_dev) + '" with "' + str(formated_drive_id) + '"')
                    if formated_drive_id == win_dev:
                        #self.debug('Found match')
                        return device
            if update:
                self.update_cygwin_windows_device_map(force_update=True)
            else:
                break
        self.debug('WARNING: Could not find cygwin device for type:"' + str(windisk_classname) + '", deviceid:' + str(drive_id))
        return ""

    def get_parsed_wmic_command_output(self, wmic_command, verbose=False):
        '''
        Attempts to parse a wmic command using "/format:textvaluelist.xsl" for key value format into a list of
        dicts.
        :param wmic_command: string representing the remote wmic command to be run
        :returns : list of dict(s) created from the parsed key value output of the command.
                   Note keys will be in lowercase

        '''
        self.debug('get_parsed_wmic_command_output, command:' + str(wmic_command))
        ret_dicts = []
        output = self.sys(wmic_command, verbose=verbose, code=0)
        newdict = {}
        for line in output:
            if not re.match(r"^\w",line):
                #If there is a blank line(s) then the previous object is complete
                if newdict:
                    ret_dicts.append(newdict)
                    newdict = {}
            else:
                splitline = line.split('=')
                key = str(splitline.pop(0)).lower()
                if len(splitline) > 1:
                    value = "=".join(str(x) for x in splitline)
                else:
                    if splitline:
                        value = splitline.pop()
                    else:
                        value = ''
                newdict[key] = value
        return ret_dicts

    def get_logicaldisk_ids(self, forceupdate=False):
        '''
        :param forceupdate: boolean, to force an update of logical disks detected on the guest. Otherwise updates are
                throttled to self.disk_update_interval
        :returns list of device ids (ie: [A:,C:,D:]
        '''
        ret = []
        self.update_disk_info(forceupdate=forceupdate)
        for disk in self.logicaldisks:
            ret.append(disk.deviceid)
        return ret

    def get_diskdrive_ids(self, drivelist=None, forceupdate=False):
        '''
        :param forceupdate: boolean, to force an update of logical disks detected on the guest. Otherwise updates are
                throttled to self.disk_update_interval
        :returns list of device ids ie: ['\\.\PHYSICALDRIVE0','\\.\PHYSICALDRIVE1,'\\.\PHYSICALDRIVE2']
        '''
        ret = []
        if not drivelist:
            self.update_disk_info(forceupdate=forceupdate)
            drivelist = self.diskdrives
        for disk in drivelist:
            ret.append(disk.deviceid)
        return ret

    def get_diskdrive_by_deviceid(self, deviceid):
        for disk in self.diskdrives:
            if disk.deviceid == deviceid:
                return disk


    def found(self, command, regex):
        """ Returns a Boolean of whether the result of the command contains the regex"""
        result = self.sys(command)
        for line in result:
            found = re.search(regex,line)
            if found:
                return True
        return False

    def assertFilePresent(self,filepath):
        '''
        Raise exception if file not found at filepath on remote guest. dirs '\' need to be represented as '\\'
        '''
        self.sys('dir ' + str(filepath), code=0)

    def assertCygwinFilePresent(self, filepath):
        self.cygwin_cmd('ls ' + str(filepath), code=0)


    def attach_volume(self, volume,  dev=None, timeout=180, overwrite=False):
        '''
        Method used to attach a volume to an instance and track it's use by that instance
        required - euvolume - the euvolume object being attached
        required - tester - the eucaops/eutester object/connection for this cloud
        optional - dev - string to specify the dev path to 'request' when attaching the volume to
        optional - timeout - integer- time allowed before failing
        optional - overwrite - flag to indicate whether to overwrite head data of a non-zero filled volume upon attach for md5
        '''
        if not isinstance(volume, EuVolume):
            volume = EuVolume.make_euvol_from_vol(volume)
        return self.attach_euvolume(volume,  dev=dev, timeout=timeout, overwrite=overwrite)


    def attach_euvolume(self, euvolume, dev=None, timeout=180, overwrite=False):
        '''
        Method used to attach a volume to an instance and track it's use by that instance
        required - euvolume - the euvolume object being attached
        required - tester - the eucaops/eutester object/connection for this cloud
        optional - dev - string to specify the dev path to 'request' when attaching the volume to
        optional - timeout - integer- time allowed before failing
        optional - overwrite - flag to indicate whether to overwrite head data of a non-zero filled volume upon attach for md5
        '''
        if not isinstance(euvolume, EuVolume):
            raise Exception("Volume needs to be of type euvolume, try attach_volume() instead?")

        self.debug('Disk drive summary before attach attempt:')
        self.print_logicaldisk_summary()
        self.print_diskdrive_summary()
        self.debug("Attempting to attach volume:"+str(euvolume.id)+" to instance:" +str(self.id)+" to dev:"+ str(dev))
        #grab a snapshot of our devices before attach for comparison purposes
        diskdrive_list_before = self.get_diskdrive_ids()
        use_serial = False
        for disk in self.diskdrives:
            if re.search('vol-', disk.serialnumber):
                use_serial = True
                break

        attached_dev = None
        start= time.time()
        elapsed = 0
        if dev is None:
            #update our block device prefix
            dev = self.get_free_scsi_dev()
        if (self.tester.attach_volume(self, euvolume, dev, pause=10,timeout=timeout)):
            if euvolume.attach_data.device != dev:
                raise Exception('Attached device:' + str(euvolume.attach_data.device) +
                                ", does not equal requested dev:" + str(dev))
            #Find device this volume is using on guest...
            euvolume.guestdev = None
            while (not euvolume.guestdev and elapsed < timeout):
                #Since all hypervisors may not support serial number info, check for an incremental diff in the
                # list of physical diskdrives on this guest.
                self.debug("Checking for volume attachment on guest, elapsed time("+str(elapsed)+")")
                diskdrive_list_after = self.get_diskdrive_ids(forceupdate=True)
                self.print_logicaldisk_summary()
                self.print_diskdrive_summary()
                self.debug("dev_list_after:"+" ".join(diskdrive_list_after))
                diff =list( set(diskdrive_list_after) - set(diskdrive_list_before) )
                if len(diff) > 0:
                    self.debug('Got Diff in drives:' + str(diff))
                    for disk in self.diskdrives:
                        if re.search('vol-', disk.serialnumber):
                            use_serial = True
                        if euvolume.id == disk.ebs_volume:
                            attached_dev = disk.deviceid
                            euvolume.guestdev = attached_dev
                            self.debug("Volume:"+str(euvolume.id)+" guest device by serialnumber:"+str(euvolume.guestdev))
                            break
                    if not use_serial:
                        attached_dev = str(diff[0])
                        euvolume.guestdev = attached_dev.strip()
                        self.debug("Volume:"+str(euvolume.id)+"found guest device by diff:"+str(euvolume.guestdev))
                    if attached_dev:
                        euvolume.guestdev = attached_dev
                        self.attached_vols.append(euvolume)
                        self.debug(euvolume.id+": Requested dev:"+str(euvolume.attach_data.device)+", attached to guest device:"+str(euvolume.guestdev))
                        break
                elapsed = int(time.time() - start)
                time.sleep(2)
            if not euvolume.guestdev or not attached_dev:
                raise Exception('Device not found on guest after '+str(elapsed)+' seconds')
        else:
            self.debug('Failed to attach volume:'+str(euvolume.id)+' to instance:'+self.id)
            raise Exception('Failed to attach volume:'+str(euvolume.id)+' to instance:'+self.id)
        if (attached_dev is None):
            self.debug("List after\n"+" ".join(diskdrive_list_after))
            raise Exception('Volume:'+str(euvolume.id)+' attached, but not found on guest'+str(self.id)+' after '+str(elapsed)+' seconds?')
        #Store the md5sum of this diskdrive in the euvolume...
        disk = self.get_diskdrive_by_deviceid(attached_dev)
        euvolume.md5len = 1024
        euvolume.md5 = self.get_dev_md5(devpath=disk.cygwin_scsi_drive, length=euvolume.md5len)
        #update the volume and instances information about the attachment...
        self.update_volume_guest_info(volume=euvolume,md5=euvolume.md5, md5len=euvolume.md5len, guestdev=euvolume.guestdev)
        self.debug('Success attaching volume:'+str(euvolume.id)+' to instance:'+self.id +
                   ', cloud dev:'+str(euvolume.attach_data.device)+', attached dev:'+str(attached_dev) +
                    ", elapsed:" + str(elapsed))
        try:
            self.rescan_disks(timeout=20)
        except Exception, e:
            self.debug('Warning. Error while trying to rescan disks after attaching volume. Error: ' + str(e))
        euvolume.printself(printmethod=self.debug)
        disk.print_self()
        return attached_dev


    def get_guest_dev_for_volume(self, volume, forceupdate=False):
        use_serial = False
        self.update_disk_info(forceupdate=forceupdate)
        for disk in self.diskdrives:
            if re.search('vol-', disk.serialnumber):
                use_serial = True
                break

        if not isinstance(volume, EuVolume):
            volume = EuVolume.make_euvol_from_vol(volume=volume, tester=self.tester)


    def get_disk_drive_by_id(self, deviceid):
        self.update_system_info()
        for disk in self.diskdrives:
            if disk.deviceid == deviceid:
                return disk
        return None


    def get_guestdevs_inuse_by_vols(self):
        retlist = []
        for vol in self.attached_vols:
            retlist.append(vol.guestdev)
        return retlist


    def get_free_scsi_dev(self, prefix=None,maxdevs=16):
        '''
        The volume attach command requires a cloud level device name that is not currently associated with a volume
        Note: This is the device name from the clouds perspective, not necessarily the guest's
        This method attempts to find a free device name to use in the command
        optional - prefix - string, pre-pended to the the device search string
        optional - maxdevs - number use to specify the max device names to iterate over.Some virt envs have a limit of 16 devs.
        '''
        d='e'
        in_use_cloud = ""
        in_use_guest = ""
        dev = None
        if prefix is None:
            prefix = self.block_device_prefix
        cloudlist=self.tester.get_volumes(attached_instance=self.id)

        for x in xrange(0,maxdevs):
            inuse=False
            #double up the letter identifier to avoid exceeding z
            if d == 'z':
                prefix= prefix+'e'
            dev = "/dev/"+prefix+str(d)
            for avol in self.attached_vols:
                if avol.attach_data.device == dev:
                    inuse = True
                    in_use_guest += str(avol.id)+", "
                    continue
            #Check to see if the cloud has a conflict with this device name...
            for vol in cloudlist:
                vol.update()
                if (vol.attach_data is not None) and (vol.attach_data.device == dev):
                    inuse = True
                    in_use_cloud += str(vol.id)+", "
                    continue
            if inuse is False:
                self.debug("Instance:"+str(self.id)+" returning available cloud scsi dev:"+str(dev))
                return str(dev)
            else:
                d = chr(ord('e') + x) #increment the letter we append to the device string prefix
                dev = None
        if dev is None:
            raise Exception("Could not find a free scsi dev on instance:"+self.id+", maxdevs:"+str(maxdevs)+"\nCloud_devs:"+str(in_use_cloud)+"\nGuest_devs:"+str(in_use_guest))


    def detach_euvolume(self, euvolume, waitfordev=True, timeout=180):
        '''
        Method used to detach detach a volume to an instance and track it's use by that instance
        required - euvolume - the euvolume object being deattached
        waitfordev - boolean to indicate whether or no to poll guest instance for local device to be removed
        optional - timeout - integer seconds to wait before timing out waiting for the volume to detach
        '''
        start = time.time()
        elapsed = 0
        found = True
        for vol in self.attached_vols:
            if vol.id == euvolume.id:
                dev = vol.guestdev
                if (self.tester.detach_volume(euvolume,timeout=timeout)):
                    if waitfordev:
                        self.debug("Cloud has detached" + str(vol.id) + ", Wait for device:"+str(dev)+" to be removed on guest...")
                        while (elapsed < timeout):
                            diskdrive_ids = []
                            try:
                                disk_drives = self.get_updated_diskdrive_info()
                                for disk in disk_drives:
                                    if dev == disk.deviceid:
                                        found = True
                                        break
                                    found = False
                                    self.debug('Diskdrive associated with ' + str(vol.id) + ' has been removed from guest.')
                                    #if device is not present remove it
                                    self.attached_vols.remove(vol)

                            except Exception, de:
                                self.debug('Warning, error getting diskdrive id during detach:' + str(de))
                            if not found:
                                try:
                                    self.rescan_disks(timeout=20)
                                except Exception, re:
                                    self.debug('Warning: Error while trying to rescan disks after detaching volume:' + str(re))
                                try:
                                    self.update_disk_info()
                                except Exception, ue:
                                    self.debug('Warning: Error while trying to update disk info:' + str(ue))
                                try:
                                    self.print_diskdrive_summary()
                                except: pass
                                self.debug('Volume:' + str(vol.id) + ', detached, and no longer found on guest at:' + str(dev))
                                vol.set_volume_detached_tags()
                                return True
                            time.sleep(10)
                            elapsed = int(time.time()-start)
                            diskdrive_ids = self.get_diskdrive_ids(drivelist=disk_drives)
                            self.debug('Current disk drives on guest:' + ",".join(str(x) for x in diskdrive_ids))
                            self.debug("Waiting for device '"+str(dev)+"' on guest to be removed.Elapsed:"+str(elapsed))

                    else:
                        self.attached_vols.remove(vol)
                        vol.set_volume_detached_tags()
                        return True
                else:
                    raise Exception("Volume("+str(vol.id)+") failed to detach from device("+str(dev)+") on ("+str(self.id)+")")

        raise Exception("Detach Volume("+str(euvolume.id)+") not found on ("+str(self.id)+")")
        return False

    def check_hostname(self):
        if not hasattr(self, 'system_info'):
            self.update_system_info()
        if hasattr(self, 'system_info') and hasattr(self.system_info, 'host_name'):
            if self.id.upper() == self.system_info.host_name.upper():
                self.debug('Hostname:' + str(self.id) + ", instance.id:" + str(self.system_info.host_name))
            else:
                raise Exception('check_hostname failed: hostname:' + str(self.system_info.host_name).upper() +
                                " != id:" + str(self.id).upper())
        else:
            raise Exception('check_hostname failed: System_info.hostname not populated')

    def get_process_list_brief(self):
        '''
        Returns a list of dicts representing the processes running on the remote guest. Each service is represented by a
        dict containing information about the service.
        '''
        cmd = "wmic process list brief /format:textvaluelist.xsl"
        return self.get_parsed_wmic_command_output(cmd)

    def get_process_list_full(self):
        '''
        Returns a list of dicts representing the processes running on the remote guest. Each service is represented by a
        dict containing information about the service.
        '''
        cmd = "wmic process list full"
        return self.get_parsed_wmic_command_output(cmd)

    def get_process_by_name(self,process_name):
        '''
        Attempts to lookup a service on the remote guest.
        param service_name: string. The name of the service to get info
        returns a dict representing the information returned from the remote guest
        '''
        cmd = 'wmic process ' + str(process_name) + ' get /format:textvaluelist.xsl'
        result = self.get_parsed_wmic_command_output(cmd)
        if result:
            return result[0]

    def get_services_list_brief(self):
        '''
        Returns a list of dicts representing the services from the remote guest. Each service is represented by a
        dict containing information about the service.
        '''
        cmd = 'wmic service list brief /format:textvaluelist.xsl'
        return self.get_parsed_wmic_command_output(cmd)

    def get_services_list_full(self):
        '''
        Returns a list of dicts representing the services from the remote guest. Each service is represented by a
        dict containing information about the service.
        '''
        cmd = 'wmic service list full'
        return self.get_parsed_wmic_command_output(cmd)

    def get_service_by_name(self,service_name):
        '''
        Attempts to lookup a service on the remote guest.
        param service_name: string. The name of the service to get info
        returns a dict representing the information returned from the remote guest
        '''
        cmd = 'wmic service ' + str(service_name) + ' get /format:textvaluelist.xsl'
        result = self.get_parsed_wmic_command_output(cmd)
        if result:
            return result[0]

    def get_memtotal_in_mb(self):
        return long(self.system_info.total_physical_memory.split()[0].replace(',',''))

    def get_memtotal_in_gb(self):
        return long(self.get_memtotal_in_mb()/1024)

    def check_ram_against_vmtype(self, pad=32):
        total_ram = self.get_memtotal_in_mb()
        self.debug('Ram check: vm_ram:' + str(self.vmtype_info.ram)
                   + "mb vs memtotal:" + str(total_ram)
                   + "mb. Diff:" + str(self.vmtype_info.ram - total_ram)
                   + "mb, pad:" + str(pad) + "mb")
        if not ((self.vmtype_info.ram - total_ram) <= pad):
            raise Exception('Ram check failed. vm_ram:' + str(self.vmtype_info.ram)
                            + " vs memtotal:" + str(total_ram) + ". Diff is greater than allowed pad:" + str(pad) + "mb")
        else:
            self.debug('check_ram_against_vmtype, passed')

    def check_ephemeral_against_vmtype(self):
        gb = self.gigabyte
        size = self.vmtype_info.disk
        ephemeral_dev = self.get_ephemeral_dev()
        block_size = self.get_blockdev_size_in_bytes(ephemeral_dev)
        gbs = block_size / gb
        self.debug('Ephemeral check: ephem_dev:'
                   + str(ephemeral_dev)
                   + ", bytes:"
                   + str(block_size)
                   + ", gbs:"
                   + str(gbs)
                   + ", vmtype size:"
                   + str(size))
        if gbs != size:
            raise Exception('Ephemeral check failed. ' + str(ephemeral_dev) + ' Blocksize: '
                            + str(gbs) + "gb (" + str(block_size) + "bytes)"
                            + ' != vmtype size:' +str(size) + "gb")
        else:
            self.debug('check_ephemeral_against_vmtype, passed')
        return ephemeral_dev

    def get_ephemeral_dev(self):
        """
        Attempts to find the block device path on this instance

        :return: string representing path to ephemeral block device
        """
        ephem_name = None
        dev_prefixs = ['s','v','xd','xvd']
        if not self.root_device_type == 'ebs':
            try:
                self.assertFilePresent('/dev/' + str(self.rootfs_device))
                return self.rootfs_device
            except:
                ephem_name = 'da'
        else:
            ephem_name = 'db'
        devs = self.get_dev_dir()
        for prefix in dev_prefixs:
            if str(prefix+ephem_name) in devs:
                return str('/dev/'+prefix+ephem_name)
        raise Exception('Could not find ephemeral device?')


    def cygwin_cmd(self, cmd, timeout=120, verbose=False, code=None):
        cmd = self.get_cygwin_path() + '\\bin\\bash.exe --login -c "' + str(cmd) + '"'
        return self.sys(cmd,timeout=timeout, verbose=verbose, code=code)

    def get_dev_md5(self, devpath, length, timeout=60):
        self.assertCygwinFilePresent(devpath)
        if length == 0:
            md5 = str(self.cygwin_cmd('md5sum ' + devpath, timeout=timeout)[0]).split(' ')[0].strip()
        else:
            md5 = str(self.cygwin_cmd("head -c " + str(length) + " " + str(devpath) + " | md5sum")[0]).split(' ')[0].strip()
        return md5


    def update_cygwin_windows_device_map(self, prefix='/dev/*', force_update=False):
        cygwin_dev_map = {}
        if not force_update:
            if self.cygwin_dev_map:
                if time.time() - self.cygwin_dev_map['last_updated'] <= 30:
                    cygwin_dev_map = self.cygwin_dev_map
        if not cygwin_dev_map:
            self.debug('Updating cygwin to windows device mapping...')
            output = self.cygwin_cmd("for DEV in " + prefix + " ; do printf $DEV=$(cygpath -w $DEV); echo ''; done",
                                     verbose=False, code=0)
            for line in output:
                if re.match(prefix, line):
                    split = line.split('=')
                    key = split.pop(0)
                    if split:
                        value = split.pop()
                    else:
                        value = ''
                    cygwin_dev_map[key]=value
            cygwin_dev_map['last_updated'] = time.time()
            self.cygwin_dev_map = cygwin_dev_map
            self.debug('Updated cygwin to windows device mapping')
        return cygwin_dev_map


    def rescan_disks(self, timeout=20):
        '''
        Attempts to rescan disks on the guest. This may help expedite updates/discovery when attaching/detaching
        volumes to the guest. This has also been found to hang post device removal so is used with a 20 second
        command timeout as the default.
        param timeout: integer. Seconds to wait on command before failing
        '''
        scriptname = 'eutester_diskpart_script'
        self.sys('(echo rescan && echo list disk ) > ' + str(scriptname), code=0)
        self.sys('diskpart /s ' + str(scriptname), code=0, timeout=timeout)


    def get_diskdrive_for_volume(self, volume):
        if not self.is_volume_attached_to_this_instance(volume):
            return None
        ret_disk = None
        for disk in self.diskdrives:
            disk.update_ebs_info()
            if disk.ebs_volume == volume.id:
                ret_disk = disk
        if not ret_disk:
            ret_disk = self.find_diskdrive_for_volume_by_serial_number(volume, force_check=True)
        if not ret_disk:
            if hasattr(volume,'md5') and volume.md5:
                ret_disk = self.find_diskdrive_for_volume_by_md5(volume, force_check=True)
        return ret_disk



    def find_diskdrive_for_volume_by_md5(self, volume, md5=None, length=None, force_check=False):
        if not force_check and not self.is_volume_attached_to_this_instance(volume):
            return None
        if not isinstance(volume, EuVolume):
            volume = EuVolume.make_euvol_from_vol(volume=volume,tester=self.tester)
        md5 = md5 or volume.md5
        if not md5:
            return None
        length = length or volume.md5len
        for disk in self.diskdrives:
            if disk.cygwin_scsi_drive:
                disk_md5 = self.get_dev_md5(disk.cygwin_scsi_drive, length=length)
                if disk_md5 == md5:
                    volume.guestdev = disk.deviceid
                    volume.md5 = disk_md5
                    volume.md5len = length
                    disk.ebs_volume = volume.id
                    return disk
        return None



    def find_diskdrive_for_volume_by_serial_number(self, volume, serial_number=None, force_check=False):
        '''
        Attempt to iterate through all the diskdrives were aware of. If a diskdrive is found with a serial_number
        associated with the volume, return that diskdrive obj..
        example serial number format: vol-81C13EA4-dev-sdg

        :param volume: volume obj to use for deriving the serial_number
        :param serial_number: string. Optional. The string representing the serial # to match.
        :returns WinInstanceDiskDrive if found, else None
        '''
        if not force_check and not self.is_volume_attached_to_this_instance(volume):
            return None
        if not serial_number:
            serial_number = volume.id + volume.attach_data.device.replace('/','-')
        for disk in self.diskdrives:
            if disk.serialnumber == serial_number:
                return disk
        return None



    def is_volume_attached_to_this_instance(self, volume):
        '''
        Attempts to look up volume state per cloud to confirm the cloud believe the state of this volume is attached
        to this instance. This does not verify the guest/hypervisor also belives the volume is attached.
        :param volume: volume obj.
        :returns boolean
        '''
        volume.update()
        if hasattr(volume, 'attach_data') and volume.attach_data and (volume.attach_data.instance_id == self.id):
            self.debug('Volume:' + str(volume.id) + " is attached to this instance: " + str(self.id) + " per cloud perspective")
            return True
        else:
            self.debug('Volume:' + str(volume.id) + " is NOT attached to this instance: " + str(self.id) + " per cloud perspective")
            return False



    def update_volume_guest_info(self, volume, md5=None, md5len=None, guestdev=None):
        if not self.is_volume_attached_to_this_instance(volume):
            raise Exception('Volume not attached to this instance')
        disk = None
        attached_volume = self.get_volume_from_attached_list_by_id(volume.id)
        if attached_volume:
            volume = attached_volume
        else:
            self.attached_vols.append(volume)
        volume.guestdev = guestdev or volume.guestdev
        if md5:
            if not md5len:
                raise Exception('Must provide md5len if providing the md5')
            volume.md5 = md5
            volume.md5len = md5len
        else:
            disk = self.get_diskdrive_for_volume(volume)
            if not disk:
                raise Exception('Could not find diskdrive for volume when attempting to update volume guest info:' + str(volume))
            volume.md5len = md5len or 1024
            volume.md5 = self.get_dev_md5(disk.cygwin_scsi_drive, volume.md5len)
        if not guestdev:
            volume.guestdev = disk.deviceid
        disk = disk or self.get_diskdrive_for_volume(volume)
        disk.update_ebs_info()
        volume.update_volume_attach_info_tags(md5=volume.md5, md5len=volume.md5len, instance_id=self.id, guestdev=volume.guestdev)
        return volume

    def get_unsynced_volumes(self, check_md5=True):
        '''
        Description: Returns list of volumes which are:
        -in a state the cloud believes the vol is no longer attached
        -the attached device has changed, or is not found.
        If all euvols are shown as attached to this instance, and the last known local dev is present and/or a local device is found with matching md5 checksum
        then the list will return 'None' as all volumes are successfully attached and state is in sync.
        By default this method will iterate through all the known euvolumes attached to this euinstance.
        A subset can be provided in the list argument 'euvol_list'.
        Returns a list of euvolumes for which a corresponding guest device could not be found, or the cloud no longer believes is attached.

        :param euvol_list: - optional - euvolume object list. Defaults to all self.attached_vols
        :param md5length: - optional - defaults to the length given in each euvolume. Used to calc md5 checksum of devices
        :param timerpervolume: -optional - time to wait for device to appear, per volume before failing
        :param min_polls: - optional - minimum iterations to check guest devs before failing, despite timeout
        :param check_md5: - optional - find devices by md5 comparision. Default is to only perform this check when virtio_blk is in use.
        '''
        bad_list = []
        retdict = self.sync_attached_volumes_with_clouds_view()
        bad_list.extend(retdict['badvols'])
        return bad_list





    def reboot_instance_and_verify(self,
                                   waitconnect=60,
                                   timeout=600,
                                   wait_for_ports=180,
                                   connect=True,
                                   checkvolstatus=False,
                                   pad=5,
                                   uptime_retries=3):
        '''
        Attempts to reboot an instance and verify it's state post reboot.
        waitconnect-optional-integer representing seconds to wait before attempting to connect to instance after reboot
        timeout-optional-integer, seconds. If a connection has failed, this timer is used to determine a retry
        connect- optional - boolean to indicate whether an ssh session should be established once the expected state has been reached
        checkvolstatus - optional -boolean to be used to check volume status post start up
        '''
        msg=""
        newuptime = None
        attempt = 0
        def get_safe_uptime():
            uptime = None
            try:
                uptime = self.get_uptime()
            except: pass
            return uptime
        self.debug('Attempting to reboot instance:'+str(self.id)+', check attached volume state first')
        uptime = self.tester.wait_for_result( get_safe_uptime, None, oper=operator.ne)
        elapsed = 0
        start = time.time()
        if checkvolstatus:
            #update the md5sums per volume before reboot
            bad_vols=self.get_unsynced_volumes()
            if bad_vols != []:
                for bv in bad_vols:
                    self.debug(str(self.id)+'Unsynced volume found:'+str(bv.id))
                raise Exception(str(self.id)+"Could not reboot using checkvolstatus flag due to unsync'd volumes")
        self.debug('Rebooting now...')
        self.reboot()
        time.sleep(waitconnect)
        try:
            self.poll_for_ports_status(ports=[3389,5589], timeout=wait_for_ports)
        except:
            self.debug('Failed to poll winrm and rdp ports after ' + str(wait_for_ports) + ' seconds, try to connect anyways...')
        timeout=timeout - int(time.time()-start)
        while (elapsed < timeout):
            self.connect_to_instance(timeout=timeout)
            #Wait for the system to provide a valid response for uptime, early connections may not
            newuptime = self.tester.wait_for_result( get_safe_uptime, None, oper=operator.ne)
            elapsed = int(time.time()-start)
            #Check to see if new uptime is at least 'pad' less than before, allowing for some pad
            if (newuptime - (uptime+elapsed)) > pad:
                err_msg = "Instance uptime does not represent a reboot. Orig:"+str(uptime)+\
                          ", New:"+str(newuptime)+", elapsed:"+str(elapsed)+"/"+str(timeout)
                if elapsed > timeout:
                    raise Exception(err_msg)
                else:
                    self.debug(err_msg)
            else:
                self.debug("Instance uptime indicates a reboot. Orig:"+str(uptime)+\
                          ", New:"+str(newuptime)+", elapsed:"+str(elapsed))
                break
        if checkvolstatus:
            badvols= self.get_unsynced_volumes()
            if badvols != []:
                for vol in badvols:
                    msg = msg+"\nVolume:"+vol.id+" Local Dev:"+vol.guestdev
                raise Exception("Missing volumes post reboot:"+str(msg)+"\n")
        self.debug(self.id+" reboot_instance_and_verify Success")


    def get_uptime(self, updateinterval=10):
            #11/18/2013, 3:15:39 PM
        if not hasattr(self, 'system_info'):
            self.update_system_info()
        splitdate = self.system_info.system_boot_time.split()
        datestring = splitdate[0]
        timestring = splitdate[1]
        ampm = splitdate[2]
        month, day, year = datestring.replace(',',"").split('/')
        hours, minutes, seconds = timestring.split(':')
        if ampm == 'PM':
            hours = int(hours) + 12
        datetimestring = str(year) + " " + \
                         str(month) + " " + \
                         str(day) + " " + \
                         str(hours) + " " + \
                         str(minutes) + " " + \
                         str(seconds)
        dt = datetime.strptime(datetimestring, "%Y %m %d %H %M %S")
        return int(time.time() - time.mktime(dt.timetuple()))






















# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
'''
Created on Oct 30, 2012
@author: clarkmatthew
Place holder for snapshot test specific convenience methods+objects to extend boto's snapshot class

'''
from boto.ec2.snapshot import Snapshot
from eutester.taggedresource import TaggedResource
import eucaops
import time



class EuSnapshot(Snapshot, TaggedResource):
    eutest_volume_md5 = None
    eutest_volume_md5len = None
    eutest_volume_zone = None
    eutest_failmsg = None
    eutest_laststatus = None
    eutest_ageatstatus = None
    eutest_cmdstart = None
    eutest_createorder = None
    eutest_cmdtime = None
    eutest_polls = None
    eutest_poll_count = None
    eutest_last_progress = None
    eutest_timeintest = None
    
        
    @classmethod
    def make_eusnap_from_snap(cls, snapshot, tester=None, cmdstart=None):
        newsnap = EuSnapshot(snapshot.connection)
        newsnap.__dict__ = snapshot.__dict__
        newsnap.eutest_volume_md5 = None
        newsnap.tester = tester
        newsnap.eutest_volume_md5len = None
        newsnap.eutest_volume_zone = None
        newsnap.eutest_volumes = []
        newsnap.eutest_failmsg = None
        newsnap.eutest_laststatus = newsnap.status
        newsnap.eutest_ageatstatus = 0 
        newsnap.eutest_cmdstart = cmdstart or eucaops.EC2ops.get_snapshot_time_started(snapshot)
        newsnap.eutest_createorder = None
        newsnap.eutest_cmdtime = None
        newsnap.eutest_polls = 0
        newsnap.eutest_poll_count = 0
        newsnap.eutest_last_progress = int(newsnap.progress.replace('%','')  or 0)
        newsnap.eutest_timeintest = 0
        return newsnap
    
    def update(self):
        super(EuSnapshot, self).update()
        self.set_last_status()
    
    def set_last_status(self,status=None):
        self.eutest_laststatus = self.status
        self.eutest_laststatustime = time.time()
        self.eutest_ageatstatus = "{0:.2f}".format(time.time() - self.eutest_cmdstart)
        
    def printself(self,title=True, printmethod=None):
        buf = "\n"
        if title:
             buf = "\n-------------------------------------------------------------------------------------------------------------------------------------\n"
             buf += str('SNAP_ID').ljust(15)+'|'+str('ORDER').center(5)+'|'+str('CMDTIME').center(8)+'|'+str('ELAPSED').center(8)+'|'+str('%').center(4)+'|'+str('STATUS').center(12)+'|'+str('SRC_VOL').center(15)+'|'+str('SRC_VOL_MD5').center(33)+'|'+str('VOLZONE').center(15)+'|'+str('INFO-MSG')+"\n"
             buf += '-----------------------------------------------------------------------------------------------------------------------\n'     
        buf += str(self.id).ljust(15)+'|'+str(self.eutest_createorder).center(5)+'|'+str(self.eutest_cmdtime).center(8)+'|'+str(self.eutest_timeintest).center(8)+'|'+str(self.eutest_last_progress).center(4)+'|'+str(self.eutest_laststatus).center(12)+'|'+str(self.volume_id).center(15)+'|'+str(self.eutest_volume_md5).center(33)+'|'+str(self.eutest_volume_zone).center(15)+'|'+str(self.eutest_failmsg)+"\n"
        buf += '-------------------------------------------------------------------------------------------------------------------------------------\n'
        if printmethod:
            printmethod(buf)
        return buf
    
        
        
    
        
        
        



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
'''
Created on Mar 7, 2012
@author: clarkmatthew
Place holder class to provide convenience for testing, modifying, and retrieving Eucalyptus cloud property information
Intention is to reduce the time in looking up property names, and values outside of the eutester test lib, etc
Note: Debug output for the tester.sys command are controled by the eutester/eucaops object 

Sample:
cat my_cloud.conf
>   192.168.1.76	CENTOS	6.3	64	REPO	[CLC WS]
>   192.168.1.77	CENTOS	6.3	64	REPO	[SC00 CC00]
>   192.168.1.78	CENTOS	6.3	64	REPO	[NC00]


    from eucaops import Eucaops
    from eutester import euproperties
    Eucaops(config_file='my_cloud.conf', password='mypassword')

    ep_mgr = euproperties.Euproperty_Manager(tester,verbose=True, debugmethod=tester.debug)


#get some storage service properties, and some property values...

    #Get/Set value from dynamic method created in Euproperty_Manager...
        san_host_prop_value = ep_mgr.get_storage_sanhost_value()
        ep_mgr.set_storage_sanhost_value('192.168.1.200')

    #Get/set value from euproperty directly...
        san_host_prop = ep_mgr.get_property('san_host', 'storage', 'PARTI00')
        san_host_prop_value = san_host_prop.get()
        san_host_prop_set('192.168.1.200'

    #Get multiple properties at once based on certain filters...
        storage_properties = ep_mgr.get_properties(service_type='storage')
        partition1_properties = ep_mgr.get_properties(partition='partition1')

'''

import types
import re
import copy

class Euproperty_Type():
    authentication = 'authentication'
    bootstrap = 'bootstrap'
    cloud = 'cloud'
    cluster = 'cluster'
    reporting = 'reporting'
    storage = 'storage'
    system = 'system'
    vmwarebroker = 'vmwarebroker'
    walrus = 'walrus'
    www = 'www'
    autoscaling = 'autoscaling'
    loadbalancing = 'loadbalancing'
    tagging = 'tagging'
    
    @classmethod
    def get_type_by_string(cls, typestring):
        try:
            if hasattr(cls, str(typestring)):
                return getattr(cls, str(typestring))
        except AttributeError, ae:
            print ('Property type:'+str(str)+" not defined, new property type?")
            raise ae


class Euproperty():
    def __init__(self, prop_mgr, property_string, service_type,  partition, name, value, mandatory=False):
        self.prop_mgr = prop_mgr
        self.service_type = Euproperty_Type.get_type_by_string(service_type)
        self.partition = partition
        self.name = name
        self.value = value
        self.property_string = property_string
        self.prop_mgr = prop_mgr
        self.lastvalue = value
        self.mandatory=mandatory
        
    def update(self):
        self.propmgr.update_property_list()

    def get(self):
        return self.value

    def set(self, value):
        return self.prop_mgr.set_property(self,value)

    def print_self(self, include_header=True, print_method=None):
        name_len = 50
        service_len = 20
        part_len = 20
        value_len = 30
        line_len = 120

        header = str('NAME').ljust(name_len)
        header += "|" + str('SERVICE TYPE').center(service_len)
        header += "|" + str('PARTITION').center(part_len)
        header += "|" + str('VALUE').center(value_len)
        header += "\n"

        out = str(self.name).ljust(name_len)
        out += "|" + str(self.service_type).center(service_len)
        out += "|" + str(self.partition).center(part_len)
        out += "|" + str(self.value).center(value_len)
        out += "\n"
        line="-"
        for x in xrange(0,line_len):
            line += "-"
        line += "\n"
        if include_header:
            ret = "\n"+line+header+line+line+out
        else:
            ret = line+out
        if print_method:
            print_method(ret)
        return ret

    
class Euproperty_Manager():
    tester = None
    verbose = False
    debugmethod = None
    
    def __init__(self, tester, verbose=False, machine=None, debugmethod=None):
        self.tester = tester
        self.debugmethod = debugmethod or tester.debug
        self.verbose = verbose
        self.work_machine =  machine or self.get_clc()
        self.access_key = self.tester.aws_access_key_id
        self.secret_key = self.tester.aws_secret_access_key
        self.service_url = 'http://'+str(self.tester.get_ec2_ip())+':8773/services/Eucalytpus'
        self.cmdpath = self.tester.eucapath+'/usr/sbin/'
        self.properties = []
        self.update_property_list()
        self.tester.property_manager = self
        
    def get_clc(self):
        return self.tester.service_manager.get_enabled_clc().machine   
        
    def debug(self,msg):
        '''
        simple method for printing debug. 
        msg - mandatory - string to be printed
        method - optional - callback to over ride default printing method 
        '''
        if (self.debugmethod is None):
            print (str(msg))
        else:
            self.debugmethod(msg)

    def show_all_authentication_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.authentication, partition=partition,debug_method=debug_method)

    def show_all_bootstrap_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.bootstrap, partition=partition,debug_method=debug_method)

    def show_all_cloud_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.cloud, partition=partition,debug_method=debug_method)

    def show_all_cluster_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.cluster, partition=partition,debug_method=debug_method)

    def show_all_reporting_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.reporting, partition=partition,debug_method=debug_method)

    def show_all_storage_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.storage, partition=partition,debug_method=debug_method)

    def show_all_system_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.system, partition=partition,debug_method=debug_method)

    def show_all_vmwarebroker_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.vmwarebroker, partition=partition,debug_method=debug_method)

    def show_all_walrus_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.walrus, partition=partition,debug_method=debug_method)

    def show_all_www_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.www, partition=partition,debug_method=debug_method)

    def show_all_autoscaling_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.autoscaling, partition=partition,debug_method=debug_method)

    def show_all_loadbalancing_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.loadbalancing, partition=partition,debug_method=debug_method)

    def show_all_tagging_properties(self,partition=None,debug_method=None):
        return self.show_all_properties(service_type=Euproperty_Type.tagging, partition=partition,debug_method=debug_method)


    def show_all_properties(self,
                            partition=None,
                            service_type=None,
                            value=None,
                            search_string=None,
                            list=None,
                            debug_method=None):
        debug_method = debug_method or self.debug
        list = list or self.get_properties(partition=partition,
                                           service_type=service_type,
                                           value=value,
                                           search_string=search_string)
        first = list.pop(0)
        buf = first.print_self(include_header=True)
        count = 1
        last_service_type = first.service_type
        for prop in list:
            count += 1
            if prop.service_type != last_service_type:
                last_service_type = prop.service_type
                print_header = True
            else:
                print_header = False
            buf += prop.print_self(include_header=print_header)
        debug_method(buf)


    def get_properties(self,
                       partition=None,
                       service_type=None,
                       value=None,
                       search_string=None,
                       force_update=False):
        self.debug('get_properties: partition:' + str(partition) \
                                                + ", service_type:" + str(service_type) \
                                                + ", value:" + str(value) \
                                                + ", force_update:" +str(force_update))
        ret_props = []
        if not self.properties or force_update:
            self.update_property_list()
        properties = copy.copy(self.properties)
        if partition and properties:
            properties = self.get_all_properties_for_partition(partition, list=properties)
        if service_type and properties:
            properties = self.get_all_properties_for_service(service_type,list=properties)
        if search_string and properties:
            properties = self.get_all_properties_by_search_string(search_string, list=properties)
        if properties:
            if value:
                for prop in properties:
                    if prop.value == value:
                        ret_props.append(prop)
            else:
                ret_props.extend(properties)
        return ret_props

    def get_property(self,name,service_type, partition, force_update=False):
        self.debug('Get Property:' + str(name))
        ret_prop = None
        list = self.get_properties(partition=partition,service_type=service_type,force_update=force_update)
        if list:
            ret_prop =  self.get_euproperty_by_name(name, list=list)
        return ret_prop

    def update_property_list(self):
        newlist = []
        self.debug("updating property list...")
        cmdout = self.work_machine.sys(self.cmdpath+'euca-describe-properties -U '+str(self.service_url)+' -I '+str(self.access_key)+' -S '+ str(self.secret_key) +' | grep PROPERTY', code=0, verbose=self.verbose)
        for propstring in cmdout:
            newlist.append(self.parse_euproperty_from_string(propstring))
        self.properties = newlist
        return newlist
        
                
    def parse_euproperty_from_string(self, propstring):
        '''
        Intended to convert a line of ouptut from euca-describe-properties into
        a euproperty. 
        :param str: line of output, example: "PROPERTY    walrus.storagemaxbucketsizeinmb    5120"
        :returns euproperty
        '''
        #self.debug('parse_euproperty_from_string, string:'+str(propstring))
        ret_service_type = None
        ret_partition = None
        splitstring = propstring.split()
        toss = splitstring.pop(0)
        ret_property_string = splitstring.pop(0)
        ret_value = " ".join(splitstring)
        #self.debug('ret_property_string:'+str(ret_property_string)+", ret_value:"+str(ret_value))
        #toss, ret_property_string, ret_value = propstring.split()
        for prop in self.properties:
            #if this property is in our list, update the value and return
            if prop.property_string == ret_property_string:
                prop.lastvalue = prop.value
                prop.value = ret_value
                return prop
        ret_name = ret_property_string
        #...otherwise this property is not in our list yet, create a new property
        #parse property string into values...
        propattrs = ret_property_string.split('.')
        partitions = self.tester.service_manager.get_all_partitions()
        for item in propattrs:
            for part in partitions:
                if part.name == item:
                    #Assume this is the partition id/name, remove it from the propattrs list
                    ret_partition = item
                    propattrs.remove(item)
                    break
        #Move along items in list until we reach a service type
        for index in xrange(0,len(propattrs)):
            try:
                ret_service_type = Euproperty_Type.get_type_by_string(propattrs[index])
                propattrs.remove(propattrs[index])
                break
            except AttributeError:
                pass
            except IndexError:
                raise Exception('No service type found for: ' + str(ret_property_string))
        #self.debug("ret_service_type: "+str(ret_service_type))

        #Store the name of the property
        ret_name = ".".join(propattrs)
        newprop = Euproperty(self, ret_property_string, ret_service_type,  ret_partition, ret_name, ret_value)
        self.create_dynamic_property_methods_from_property(newprop)
        return newprop

    def create_dynamic_property_methods_from_property(self, euproperty):
        method_name_string = str(euproperty.service_type).replace('.','_') + "_" + str(euproperty.name).replace('.','_')
        set_method_name = "set_" + str(method_name_string) + "_value"
        get_method_name = "get_" + str(method_name_string) + "_value"

        get_method_doc = "Attempts to get property: " + str(method_name_string) \
                                                      + "\nparam partitions: partition for this property"
        set_method_doc = "Attempts to set property: " + str(method_name_string) \
                                                      + "\nparam partitions: partition for this property"
        prop_mgr=self
        #self.debug('Creating dynamic methods for property:'+str(method_name_string))
        #Add a set method for this property to this euproperty manager

        if not hasattr(self,set_method_name):
            def set_method(self, value, partition=None):
                self.debug('Starting set Method for property:' + str(method_name_string))
                service_type = euproperty.service_type
                prop_name = euproperty.name
                try:
                    prop = self.get_property(name=prop_name,service_type=service_type,partition=partition)
                except IndexError:
                    raise Exception('Property not found. name:' + str(prop_name) \
                                    + ', type:' + str(service_type) \
                                    + ', partition:' + str(partition))
                return self.set_property(prop,value)
            setattr(self, set_method_name, lambda value: set_method(self,value,partition=euproperty.partition))
            new_set_method = getattr(self, set_method_name)
            new_set_method.__doc__ = set_method_doc
            new_set_method.__name__ = set_method_name

        #Add a get method for this property to this euproperty manager
        if not hasattr(self,get_method_name):
            def get_method(self, partition=None):
                service_type = euproperty.service_type
                prop_name = euproperty.name
                self.debug("get method:"+str(prop_name)+", Partition:"+str(partition)+", service_type:"+str(service_type))
                try:
                    prop = self.get_property(name=prop_name,service_type=service_type,partition=partition)
                except IndexError:
                    raise Exception('Property not found. name:' + str(prop_name) \
                                    + ', type:' + str(service_type) \
                                    + ', partition:' + str(partition))
                if not prop:
                    raise Exception('property:'+str(prop_name) + ", not found for partition:"+str(partition))
                return prop.value
            setattr(self, get_method_name, lambda partition=None: get_method(self,partition=partition))
            new_get_method = getattr(self, get_method_name)
            new_get_method.__name__ = get_method_name
            new_get_method.__doc__ =  get_method_doc


    def get_euproperty_by_name(self,name, list=None):
        props = []
        list = list or self.properties
        for property in list:
            if property.name == name:
                return property
        raise EupropertyNotFoundException('Property not found by name:'+str(name))
        
    def get_all_properties_for_partition(self, partition, list=None, verbose=False):
        self.debug('Get all properties for partition:'+str(partition))
        props = []
        list = list or self.properties
        for property in list:
            if property.partition == partition:
                if verbose:
                    self.debug('property:'+str(property.name)+", prop.partition:"+str(property.partition) +",partition:"+str(partition))
                props.append(property)
        self.debug('Returning list of len:'+str(len(props)))
        return props

    def get_all_properties_for_service(self,service, list=None):
        props = []
        list = list or self.properties
        for property in list:
            if property.service_type == service:
                props.append(property)
        return props

    def get_all_properties_by_search_string(self,search_string, list=None):
        props = []
        list = list or self.properties
        for property in list:
            if re.search(search_string,property.property_string):
                props.append(property)
        return props

    def set_property(self, property, value):
        if isinstance(property,Euproperty):

            return self.set_property_by_property_string(property.property_string, value)
        else:
            return self.set_property_by_property_string(str(property), value)

    def set_property(self,  property, value):
        '''
        Sets the property 'prop' at eucaops/eutester object 'tester' to 'value'    
        Returns new value  
        prop - mandatory - string representing the property to set
        value - mandatory - string representing the value to set the property to
        eucaops - optional - the eucaops/eutester object to set the property at
        '''
        value = str(value)
        if not isinstance(property,Euproperty):
            try:
                property = self.get_all_properties_by_search_string(property)
                if len(property) > 1:
                    raise Exception('More than one euproperty found for property string:' +str(property))
                else:
                    property = property[0]
            except Exception, e:
                raise Exception('Could not fetch property to set. Using string:' +str(property))
        property.lastvalue = property.value
        self.debug('Setting property('+property.property_string+') to value:'+str(value))
        ret_string = self.work_machine.sys(self.cmdpath+'euca-modify-property -U '+str(self.service_url)+' -I '+str(self.access_key)+' -S '+ str(self.secret_key) +' -p '+property.property_string+'='+str(value), code=0)[0]

        if (ret_string):
            ret_value= str(ret_string).split()[2]
        else:
            raise EupropertiesException("set_property output from modify was None")
        
        if (ret_value != value):
            raise EupropertiesException("set property("+property.property_string+") to value("+str(value)+") failed.Ret Value ("+str(ret_value)+")\nRet String\n"+ret_string)
        property.value = ret_value
        return ret_value
        
    def reset_property_to_default(self, prop):
        '''
        Sets a property 'prop' at eucaops/eutester object 'eucaops' to it's default value
        Returns new value
        prop - mandatory - string representing the property to set
        ucaops - optional - the eucaops/eutester object to set the property at
        '''

        if not isinstance(prop, Euproperty):
               prop = self.get_all_properties_by_search_string(prop)[0]
        property_string = prop.property_string
        prop.lastvalue = prop.value
        ret_string = str(self.work_machine.sys(self.cmdpath+'euca-modify-property -U '+str(self.service_url)+' -I '+str(self.access_key)+' -S '+ str(self.secret_key) +' -r '+str(property_string),code=0)[0])
        ret_value= ret_string.split()[2]
        self.debug('Reset property('+str(prop.name)+') to default value('+str(ret_value)+')')
        return ret_value
        
    def get_property_default_value(self, prop, ireadthewarning=False):
        '''
        Note: This hack method is intrusive! It will briefly reset the property
        This is a temporary method to get a properties default method
        prop - mandatory - string, eucalyptus property
        ireadthewarning - mandatory - boolean, to warn user this method is intrusive 
        '''
        if (ireadthewarning is False):
            raise EupropertiesException("ireadthewarning is set to false in get_property_default_value")
    
        original = prop.get()
        default = self.reset_property_to_default(prop)
        prop.set(original)
        return default
    
        
        
        
    '''
    ########################################################################
    STORAGE RELATED PROPERTY METHODS...
    ########################################################################
    '''

    '''
    #PARTI00.storage.maxvolumesizeingb
    def get_storage_maxvolumesizeingb(self,  zone='PARTI00'):
        value = self.get_property_value(  zone+'.storage.maxvolumesizeingb')
        return value
    
    def set_storage_maxvolumesizeingb(self, value, zone='PARTI00'):
        return self.set_property(  zone+'.storage.maxvolumesizeingb', value)
    
    #PARTI00.storage.maxtotalvolumesizeingb 
    def get_storage_maxtotalvolumesizeingb(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.maxtotalvolumesizeingb')
        return value
    
    def set_storage_maxtotalvolumesizeingb(self, value,  zone='PARTI00'):
        value = self.set_property(  zone+'.storage.maxtotalvolumesizeingb', value)
        return value
    
    #PARTI00.storage.zerofillvolumes 
    def get_storage_zerofillvolumes(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.zerofillvolumes')
        return value
    
    def set_storage_zerofillvolumes(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.zerofillvolumes', value)
        
    #PARTI00.storage.volumesdir 
    def get_storage_volumesdir(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.volumesdir')
        return value
    
    def set_storage_volumesdir(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.volumesdir', value)
        
    #PARTI00.storage.tid
    def get_storage_tid(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.tid')
        return value
    
    def set_storage_tid(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.tid', value)
    
    #PARTI00.storage.storeprefix
    def get_storage_storeprefix(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.storeprefix')
        return value
    
    def set_storage_storeprefix(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.storeprefix', value)
    
    #PARTI00.storageinterface
    def get_storageinterface(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.storageinterface')
        return value
    
    def set_storageinterface(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.storageinterface', value)
    
    #PARTI00.storage.snappercent
    def get_storage_snappercent(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.snappercent')
        return value
    
    def set_storage_snappercent(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.snappercent', value)
        
    #PARTI00.storage.shouldtransfersnapshots
    def get_storage_shouldtransfersnapshots(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.shouldtransfersnapshots')
        return value
    
    def set_storage_shouldtransfersnapshots(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.shouldtransfersnapshots', value)
        
    #PARTI00.storage.sanuser
    def get_storage_sanuser(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.sanuser')
        return value
    
    def set_storage_sanuser(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.sanuser', value)
    
    #PARTI00.storage.sanpassword
    def get_storage_sanpassword(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.sanpassword')
        return value
    
    def set_storage_sanpassword(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.sanpassword', value)
        
    #PARTI00.storage.sanhost
    def get_storage_sanhost(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.sanhost')
        return value
    
    def set_storage_sanhost(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.sanhost', value)
        
    #PARTI00.storage.minornumber
    def get_storage_minornumber(self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.minornumber')
        return value
    
    def set_storage_minornumber(self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.minornumber', value)
    
    #PARTI00.storage.majornumber 
    def get_storage_majornumber (self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.majornumber ')
        return value
    
    def set_storage_majornumber (self, value,  zone='PARTI00'):
        return self.set_property(  zone+'.storage.majornumber', value)
        
    #PARTI00.storage.dasdevice 
    def get_storage_dasdevice (self,   zone='PARTI00'): 
        value = self.get_property_value(  zone+'.storage.dasdevice')
        return value
    
    def set_storage_dasdevice (self, value,  zone='PARTI00'):
        if not isinstance(value, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.dasdevice ', value)
        
    def get_storage_activestoragegroupcleaning(self, zone='PARTI00'): #false
        return self.get_property_value(  zone+'.storage.activestoragegroupcleaning')
    
    def set_storage_activestoragegroupcleaning(self, boolean,  zone='PARTI00'):
        if not isinstance(boolean, types.BooleanType):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.activestoragegroupcleaning', boolean) 
    
    def get_storage_chapuser(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.chapuser')
    
    def set_storage_chapuser(self, string,  zone='PARTI00'):    #e4c6-1000
        if not isinstance(string, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.chapuser')
    
    def get_storage_blockstoragemanager(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.blockstoragemanager')
    
    def set_storage_blockstoragemanager(self, string,  zone='PARTI00'):    #e4c6-1000
        if not isinstance(string, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.blockstoragemanager')
    
    def get_storage_clipath(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.clipath')
    
    def set_storage_clipath(self, path,  zone='PARTI00'):    #/opt/Navisphere/bin/naviseccli
        if not isinstance(path, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.clipath')
        
    def get_storage_clonestoragegroup(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.clonestoragegroup')
   
    def set_storage_clonestoragegroup(self, string,  zone='PARTI00'):    #eucalyptus_clonelungroup
        if not isinstance(string, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.clonestoragegroup')
        
    def get_storage_loginscope(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.loginscope')
    
    def set_storage_loginscope(self, int,  zone='PARTI00'):    #0
        if not isinstance(int, types.IntType):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.loginscope')
        
    def get_storage_lunspallocator(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.lunspallocator')
    
    def set_storage_lunspallocator(self, string,  zone='PARTI00'):    #EmcVnxLunSpAllocatorByVolIdHash
        if not isinstance(string, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.lunspallocator')
        
    def get_storage_ncpaths(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.ncpaths')
    
    def set_storage_ncpaths(self, string,  zone='PARTI00'):    #iface0:192.168.25.182,iface1:10.109.25.186
        if not isinstance(string, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.ncpaths')
        
    def get_storage_quiescetimeseconds(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.quiescetimeseconds')
    
    def set_storage_quiescetimeseconds(self, int,  zone='PARTI00'):    #10
        if not isinstance(int, types.IntType):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.quiescetimeseconds')
        
    def get_storage_scpaths(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.scpaths')
    
    def set_storage_scpaths(self, string,  zone='PARTI00'):    #iface0:192.168.25.182,iface1:10.109.25.186
        if not isinstance(string, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.scpaths')
        
    def get_storage_storagepool(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.storagepool')
    
    def set_storage_storagepool(self, int,  zone='PARTI00'):    #0
        if not isinstance(int, types.IntType):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.storagepool')
        
    def get_storage_syncrate(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.syncrate')
    
    def set_storage_syncrate(self, string,  zone='PARTI00'):    #high
        if not isinstance(string, types.StringTypes):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.syncrate')
        
    def get_storage_timeoutinmillis(self, zone='PARTI00'):
        return self.get_property_value(  zone+'.storage.timeoutinmillis')
   
    def set_storage_timeoutinmillis(self, int,  zone='PARTI00'):    #<unset
        if not isinstance(int, types.IntType):
            raise Exception("Incorrect type passed")
        return self.set_property(  zone+'.storage.timeoutinmillis')

        
    '''


    ########################################################################
    #WALRUS RELATED PROPERTY METHODS...
    ########################################################################
    '''
    #walrus.storagemaxtotalsnapshotsizeingb 
    def get_walrus_storagemaxtotalsnapshotsizeingb(self, tester=None):
        value = self.get_property_value(  'walrus.storagemaxtotalsnapshotsizeingb')
        return value
    def set_walrus_storagemaxtotalsnapshotsizeingb(self, value, tester=None):
        return self.set_property(  'walrus.storagemaxtotalsnapshotsizeingb', value)
    '''
    
class EupropertiesException(Exception):
    def __init__(self, value):
        self.value = value
    def __str__ (self):
        return repr(self.value)

class EupropertyNotFoundException(Exception):
    def __init__(self, value):
        self.value = value
def __str__ (self):
    return repr(self.value)

    
    


#!/usr/bin/python
#
# Author:       Vic and Graziano
# Description:  Eutester test case to reproduce bug 737335
#               Creates an instance of an image, and removes all permissions from the instance.
#               If there are still permissions after the command to remove all of them has executed, 
#               the test fails; otherwise, tries to run the image.
#    (https://bugs.launchpad.net/eucalyptus/+bug/737335)            
#

from eutester import Eutester
import argparse

if __name__ == '__main__':
    ### Parse args
    parser = argparse.ArgumentParser(description='Test for Eucalyptus Launchpad bug-737335')
    parser.add_argument("--credpath", dest="credpath",
                      help="The path where to find eucarc (default ~/.euca)", default="~/.euca") 
    parser.add_argument("--userId", dest="userId", 
                      help="userId to use to run test", default="admin") 
    args = parser.parse_args()
    
    # create Eutester object
    tester = Eutester( credpath=args.credpath )
    
    # find images owned by the user
    emi_list = tester.sys("euca-describe-images -o self | grep emi- | head -1 | awk '{print $2}'")
    
    if emi_list == []:
        tester.fail("Did not find any images owned by self")
        tester.do_exit()
    
    # grab an emi
    emi = emi_list[0]
    
    # reset permission
    tester.sys("euca-reset-image-attribute --launch-permission " + emi)
    
    # remove all permission
    tester.sys("euca-modify-image-attribute --launch-permission -r all " + emi)
    tester.sys("euca-modify-image-attribute --launch-permission -r " + args.userId + " " + emi)
    
    # check permission are empty
    if tester.sys("euca-describe-image-attribute -l " + emi) != []:
        tester.fail("Image still has attributes")
        tester.do_exit()
    
    # try to run it
    try:
        image = tester.get_emi(emi.rstrip())
        reservation = tester.run_instance(image)
        tester.fail("Was allowed to run instance")
        tester.terminate_instances(reservation)
    except Exception,e:
        tester.test_name("COMPLETED TEST SUCCESSFULLY!!!!")

    exit(0)



# Software License Agreement (BSD License)
#
# Copyright (c) 2009-2011, Eucalyptus Systems, Inc.
# All rights reserved.
#
# Redistribution and use of this software in source and binary forms, with or
# without modification, are permitted provided that the following conditions
# are met:
#
#   Redistributions of source code must retain the above
#   copyright notice, this list of conditions and the
#   following disclaimer.
#
#   Redistributions in binary form must reproduce the above
#   copyright notice, this list of conditions and the
#   following disclaimer in the documentation and/or other
#   materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
# AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
# ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
# LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
# CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
# SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
# INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.
#
# Author: matt.clark@eucalyptus.com
  
from eucaops import Eucaops
from eutester.eutestcase import EutesterTestCase
from eutester import Eutester
from eutester import machine
from testcases.cloud_user.images.imageutils import ImageUtils
from testcases.cloud_user.ebs.ebstestsuite import TestZone
from testcases.cloud_user.images.windows import windowsproxytests
from eutester.euvolume import EuVolume
import socket
import os
import time
import re
from datetime import datetime

class WindowsTests(EutesterTestCase):
    
    def __init__(self,
                 #setup eutester and test environment...
                 tester=None, 
                 config_file=None,
                 password=None, 
                 credpath=None, 
                 eof=True,
                 #Information on where to do image bundle work, and related timeouts
                 destpath=None,  
                 time_per_gig = 300,
                 inter_bundle_timeout=120, 
                 upload_timeout=0,
                 work_component=None,
                 component_credpath=None,
                 bucketname = None,
                 #Test env resources...
                 group = None,
                 testvolumes = [],
                 testvolcount = 2,
                 keypair=None, 
                 zone = None,
                 url=None,
                 user_data=None,
                 emi=None,
                 private_addressing=False,
                 instance_password = None,
                 instance_keypath = None,
                 vmtype='m1.xlarge',
                 emi_location=None,
                 image_path=None, #note this must be available on the work_component
                 instance=None,
                 win_proxy_hostname = None, 
                 win_proxy_username = 'Administrator',
                 win_proxy_password = None, 
                 win_proxy_keypath = None,
                 clean_on_exit=False,
                 authports=['tcp:3389','tcp:80','tcp:443', 'tcp:5985', 'tcp:5986']
                 ):
        """
        Definition:
        This class is intended to hold most of the methods and state(s) needed to run an array of Windows Instance
        related tests. Most TestUnits will use a subset of these methods to define a testcase/operation. See sample
        testcases scripts which reference this class for examples and use cases.

        :param tester: eutester object
        :param config_file: eutester config file
        :param credpath: path to cloud credentials/eucarc
        :param password: password used to access remote components
        :param eof: boolean, end on failure
        :param destpath: path on 'work component' in which to peform work. ie: where to download and bundle an img.
        :param time_per_gig: Time in seconds to be used for image related timeouts
        :param inter_bundle_timeout: Time to wait between bundle operation. Mainly used to detect hung operations
        :param upload_timeout: Time to wait for the upload portion of image operation
        :param work_component: The component or machine in which work is to be executed on, ie download, bundles, etc.
        :param component_credpath: The path on the 'work_component' in which to find creds. ie for tools exectuted remotely
        :param bucketname: Bucketname to be used as a global for operations in this test.
        :param group: Security group to use for this test
        :param testvolumes: List of volumes that are intended to be used in by this obj's test(s)
        :param testvolcount: Number of volumes to be used for ebs related tests defined in this test obj
        :param keypair: keypair ot be used for this test(s)
        :param zone: zone/cluster to be used to executue these tests
        :param url: url to use for fetching remote images to be built into EMI(s) for this test
        :param user_data: Any (instance) user data to be used for in this test
        :param emi: emi to be used for this test
        :param private_addressing: boolean, used to run instances w/o a public ip
        :param instance_password: password used for accessing instance(s) within this test
        :param instance_keypath: keypath used to access instance(s) within this test
        :param vmtype: type of vm to use when running instance(s) in this test, ie m1.xlarge
        :param emi_location: string used to find an existing EMI by the EMI's location-string
        :param image_path: path to an image on the local machine or work component
        :param instance: existing instance to use within this test
        :param win_proxy_hostname: The ip or FQDN of the machine used to proxy powershell and ldap tests against
        :param win_proxy_username: The user name for ssh login on the machine used to  proxy powershell and ldap tests against
        :param win_proxy_password: The password for ssh login on the machine used to  proxy powershell and ldap tests against
        :param win_proxy_keypath: The keypath for ssh login on the machine used to  proxy powershell and ldap tests against
        :param authports: What ports should be authorized within security group for testing
        """
        self.setuptestcase()
        if tester is None:
            self.tester = Eucaops( config_file=config_file,password=password,credpath=credpath)
        else:
            self.tester = tester
        self.tester.exit_on_fail = eof
        self.instance = instance
        if self.instance:
            self.instance = self.tester.get_instances(idstring=str(instance))[0]
        self.instance_keypath = instance_keypath
        self.destpath = destpath or '/tmp'
        self.bucketname = bucketname
        self.component = work_component 
        self.component_credpath = component_credpath
        self.time_per_gig = time_per_gig
        self.credpath=credpath or self.tester.credpath
        self.url = url
        self.upload_timeout = upload_timeout
        self.inter_bundle_timeout = inter_bundle_timeout
        self.iu = ImageUtils(tester=self.tester, destpath=self.destpath, time_per_gig=self.time_per_gig, work_component=self.component)
        self.group = group 
        self.keypair = keypair
        self.zonelist = []
        self.emi = self.tester.get_emi(emi)
        self.image_path = image_path
        self.emi_location = emi_location
        if zone is not None:
            self.zonelist.append(zone)
        self.testvolcount = testvolcount
        self.testvolumes = testvolumes
        self.authports=authports
        self.instance_password = instance_password
        self.vmtype= vmtype 
        self.user_data = user_data
        self.private_addressing = private_addressing
        #timeout for run instance command/test
        self.run_timeout=780 
        #setup zone list
        self.setupWindowsZones()
        #setup windows proxy 
        self.win_proxy_hostname = win_proxy_hostname
        self.proxy = None
        if win_proxy_hostname is not None:
            self.setup_proxy(win_proxy_hostname,
                             proxy_keypath = win_proxy_keypath,
                             proxy_username = win_proxy_username,
                             proxy_password = win_proxy_password,
                             debugmethod = lambda msg: self.debug(msg, traceback=2)
                             )
        self.setup_test_env()
        if self.instance and self.proxy:
                self.update_proxy_instance_data()
    @Eutester.printinfo
    def setup_proxy(self, proxy_hostname, proxy_keypath=None, proxy_username=None, proxy_password=None, debugmethod=None):
        debugmethod = debugmethod or (lambda msg: self.debug(msg, traceback=3))
        proxy = windowsproxytests.WindowsProxyTests(proxy_hostname, 
                                                         proxy_keypath = proxy_keypath,
                                                         proxy_username = proxy_username,
                                                         proxy_password = proxy_password,
                                                         debugmethod = debugmethod,
                                                         )
        self.proxy = proxy
        self.proxy.win_instance = self.instance
        try:
            self.test_get_windows_instance_password()
        except: pass
        self.proxy.win_password = self.instance_password
        self.proxy.win_keypath = self.instance_keypath
        return proxy
    
    def update_proxy_info(self):
        if not self.proxy:
            raise Exception("Proxy needs to be setup before it can be updated")
        self.proxy.win_instance = self.instance
        try:
            self.test_get_windows_instance_password()
        except: pass
        self.proxy.win_password = self.instance_password
        self.proxy.win_keypath = self.instance_keypath

    def setup_test_env(self):
        self.setupWindowsKeypair()
        self.setupWindowsSecurityGroup()
        self.setupWindowsZones()
        
    def get_images_by_location(self, match='windows'):
        retlist=[]
        for image in self.tester.ec2.get_all_images():
            if re.search(match,image.location):
                retlist.append(image)
        return retlist



    def update_proxy_instance_data(self, win_instance=None, instance_password=None ):
        if self.proxy is None:
            return
        self.proxy.win_instance = win_instance or self.instance
        try:
            password = instance_password or self.instance_password
            if password is None:
                password = self.test_get_windows_instance_password()
            self.proxy.win_password = password
        except Exception, e:
            raise Exception('Warning: update_proxy_instance_data: Could not get instance password, err:'+str(e))
        
        
            
    def setupWindowsSecurityGroup(self, portlist=None):
        portlist = portlist or self.authports
        #Setup our security group for later use...
        if self.group is None:
            
            group_name='WindowsTestGroup'
            try:
                self.group = self.tester.add_group(group_name)
                self.setupWindowsGroupAuthorization(portlist=portlist)
                #enable windows RDP port
            except Exception, e:    
                raise Exception("Error when setting up group:"+str(group_name)+", Error:"+str(e)) 
         
    def setupWindowsGroupAuthorization(self,portlist=None):
        '''
        Where format for adding port in list is: 'protocol:port', example: tcp, port 80 = 'tcp:80'
        '''
        authports = portlist or self.authports
        for p in authports:
            protocol,port = str(p).split(':')
            self.tester.authorize_group_by_name(self.group.name,protocol=str(protocol),port=int(port)) 
            
        
    def setupWindowsKeypair(self):
        #Setup the keypairs for later use
        try:
            if self.keypair is None:
                keys = self.tester.get_all_current_local_keys() 
                if keys != []:
                    self.keypair = keys[0]
                else:
                    self.keypair = keypair = self.tester.add_keypair('windows_test_key-' + str(time.time()))
            else:
                return
        except Exception, ke:
            raise Exception("Failed to find/create a keypair, error:" + str(ke))
        
           
        
    def test_get_windows_instance_password(self, instance=None, privkeypath=None):
        instance =instance or self.instance
        privkeypath = privkeypath or self.tester.verify_local_keypath(instance.key_name) 
        password = self.tester.get_windows_instance_password(instance, private_key_path = privkeypath)
        self.instance_password = password
        return password
        
    def setupWindowsZones(self):
        
        #create some zone objects and append them to the zonelist
        if self.zonelist == []:
            for zone in self.tester.service_manager.partitions.keys():
                partition = self.tester.service_manager.partitions.get(zone)
                tzone = TestZone(partition)
                self.zonelist.append(tzone)
                self.multicluster=True
        else:
            return
                
    def setup_windows_test_volumes(self):
        '''
        Description:
            Attempts to confirm that each zone provided has the minimum number of volumes in it. 
            If volumes were provided in __init__ then this method will attempt to sort those into the zone. 
            Volumes will be created if the testzones do not have the amount specified. 
        Failure:
            Failed to create volumes for use in windows test
        '''
        if len(self.zonelist) and len(self.zonelist[0].volumes):
            self.debug('setupWindowsTestVolumes has already been ran')
            return
        #sort any provided volumes into zone
        for volume in self.testvolumes:
            if volume.status == 'available':
                if type(volume) != EuVolume:
                    volume = EuVolume.make_euvol_from_vol(volume)
                for zone in self.zonelist:
                    if volume.zone == zone.name:
                        zone.volumes.append(volume)
        #make sure each zone has the correct amount of volumes to perform the tests
        for zone in self.zonelist:
            while len(zone.volumes) < self.testvolcount:
                volume = EuVolume.make_euvol_from_vol(self.tester.create_volume(zone,timepergig=180))
                zone.volumes.append(volume)
            
        
                
    def create_vols_per_zone(self, zonelist=None, volsperzone=2, size=1, snapshot=None, timepergig=300):
        testmsg =   """
                    Intention of this test is to verify creation of volume(s) per zone given.
                    Upon successful creation the volumes will be appended to a volumes list
                    for the zone it was created in. 
                    These volumes may be later used if in later ebstests suite tests. 
                    """    
        testmsg = testmsg + "variables provided:\nzonelist:"+str(zonelist)+"\nvolsperzone:"+str(volsperzone)+"\nsize:"+str(size)+"\nsnapshot:"+str(snapshot)
        
        self.startmsg(testmsg)
        if zonelist is None:
            zonelist = self.zonelist
        for testzone in zonelist:
            zone = testzone.name
            for x in xrange(0,volsperzone):
                vol = EuVolume.make_euvol_from_vol(self.tester.create_volume(zone, size=size, snapshot=snapshot,timepergig=timepergig))
                testzone.volumes.append(vol)
                self.debug('create_vols_per_zone created  vol('+str(x)+') zone:'+str(zone)+' vol:'+str(vol.id))
            
        self.endsuccess() 
    
    def create_windows_emi_from_url(self,
                                      url=None, 
                                      component = None, 
                                      component_credpath = None,
                                      bucketname = None, 
                                      destpath = None, 
                                      inter_bundle_timeout = None, 
                                      upload_timeout = None,
                                      virtualization_type = None,
                                      wget_user = None,
                                      wget_password = None,
                                      time_per_gig = None,
                                      ):
        '''
        Attempts to download (wget), bundle, upload and register a windows image at 'url' 
        Work is done on a given machine and requires euca2ools present on that machine. 
        Returns the emi of the registered image
        '''
        return self.iu.create_emi_from_url(url = (url or self.url), 
                                           component = (component or self.component), 
                                           bucketname = (bucketname or self.bucketname), 
                                           component_credpath = (component_credpath or self.component_credpath), 
                                           destination = (destpath or self.destpath),
                                           interbundle_timeout = (inter_bundle_timeout or self.inter_bundle_timeout), 
                                           upload_timeout = (upload_timeout or self.upload_timeout),
                                           destpath = (destpath or self.destpath),
                                           virtualization_type=virtualization_type,
                                           wget_user = (wget_user), 
                                           wget_password = (wget_password),   
                                           time_per_gig = (time_per_gig or self.time_per_gig) )
        
    def create_windows_emi_from_file(self,
                                     image_file_path,
                                     component = None, 
                                     component_credpath = None,
                                     bucketname = None, 
                                     destpath = None, 
                                     inter_bundle_timeout = None, 
                                     upload_timeout = None,
                                     time_per_gig = None,
                                     ):
        '''
        Definition: Attempts bundle, upload and register a windows image on component filesystem at fpath.  
        Work is done on a given machine and requires euca2ools present on that machine. 
        Returns the emi of the registered image
        '''
        return self.iu.create_emi_from_url(
                                           url = None,
                                           component = (component or self.component), 
                                           bucketname = (bucketname or self.bucketname), 
                                           component_credpath = (component_credpath or self.component_credpath), 
                                           destination = (destpath or self.destpath),
                                           interbundle_timeout = (inter_bundle_timeout or self.inter_bundle_timeout), 
                                           upload_timeout = (upload_timeout or self.upload_timeout),
                                           destpath = (destpath or self.destpath),
                                           filepath = image_file_path,
                                           time_per_gig = (time_per_gig or self.time_per_gig) )
        
    def test_run_windows_emi(self,
                      emi=None, 
                      zone=None,
                      keypair=None,
                      type=None, 
                      group=None, 
                      min=1, 
                      max=1,
                      user_data=None,
                      private_addressing=None,
                      timeout=None):
        '''
        Description: Attempts to return a reservation of running emi instances run with the provided parameters.
        '''
        emi = emi or self.emi
        zone = zone or self.zonelist[0] if self.zonelist else None
        keypair = keypair or self.keypair
        type = type or self.vmtype or 'm1.xlarge'
        group = group or self.group
        user_data = user_data or self.user_data
        private_addressing = private_addressing if private_addressing is not None else self.private_addressing
        timeout = timeout or self.run_timeout
        if not emi:
            raise Exception('test_run_windows_emi, no emi provided. ')
        res = self.tester.run_instance(emi, 
                                 keypair = keypair or self.keypair.name, 
                                 group = group or self.group, 
                                 type = type, 
                                 zone=zone, 
                                 min=min,
                                 max=max,
                                 user_data=user_data,
                                 private_addressing=private_addressing,
                                 is_reachable=False,
                                 timeout=timeout)
        
        
        self.instance = res.instances[0]
        self.debug('test_run_windows_emi, setting test instance to:'+str(self.instance.id))
        self.debug('')
        return res
        
        
    
    def test_rdp_port(self, ip=None, port=3389, timeout=10):
        '''
        Description: Attempts to test that the host is accepting tcp connections to the RDP port
        '''
        return self.test_port_status(ip=ip, port=port, timeout=timeout)
    
    
    def scan_port_range(self, start,stop,ip=None,timeout=1, tcp=True):
        ip = ip or self.instance.public_dns_name 
        return self.tester.scan_port_range(ip, int(start),int(stop), timeout=int(timeout), tcp=tcp)
    
    def test_port_status(self, port, ip=None, timeout=5, tcp=True, verbose=True):
        ip = ip or self.instance.public_dns_name 
        return self.tester.test_port_status(ip, port, timeout=timeout, tcp=tcp, verbose=verbose)

    def test_poll_for_rdp_port_status(self, instance=None,interval=10,socktimeout=5,timeout=180, waitforboot=120):
        instance = instance or self.instance
        if not instance:
            raise Exception("No instance available to test with, please add to windowstests or provide to method")
        #Make sure some time has passed before we test on the guest side before running guest test...
        attached_seconds = self.tester.get_instance_time_launched(instance)
        sleeptime =  0 if attached_seconds > waitforboot else (waitforboot - attached_seconds)
        self.debug("Instance was launched "+str(attached_seconds)+" seconds ago, waiting:"+str(sleeptime)+" for instance to boot")
        time.sleep(sleeptime)
        ip = instance.public_dns_name
        return self.test_poll_for_port_status(3389, ip=ip, interval=interval, socktimeout=socktimeout, timeout=timeout)
    
    def test_wait_for_instance_boot(self,instance=None,waitforboot=420):
        instance = instance or self.instance
        boot_seconds = self.tester.get_instance_time_launched(instance)
        sleeptime =  0 if boot_seconds > waitforboot else (waitforboot - boot_seconds)
        self.debug("Instance was launched "+str(boot_seconds)+" seconds ago, waiting:"+str(sleeptime)+" for instance to boot")
        start = time.time()
        elapsed = 0
        print "Waiting for Windows to fully boot:",
        while elapsed < sleeptime:
            print "Waiting for Windows to fully boot:"+str(sleeptime-elapsed),
            time.sleep(5)
            elapsed=int(time.time()-start)
        self.debug("test_wait_for_instance_boot: done waiting, instance up for "+str(waitforboot)+" seconds") 
    
    def test_poll_for_port_status(self, port, ip=None, interval=10, socktimeout=5, timeout=180):
        ip = ip or self.instance.public_dns_name
        start = time.time()
        elapsed = 0 
        attempt = 0
        while elapsed < timeout:
            attempt +=1 
            self.debug('test_poll_for_port_status:'+str(attempt))
            if elapsed < timeout:
                try:
                    self.test_rdp_port(ip=ip, port=port, timeout=5)
                    return
                except socket.error, se:
                    self.debug('test_port_status failed socket error:'+str(se[0]))
                    #handle specific errors here, for now just for debug...
                    ecode=se[0]
                    if ecode == socket.errno.ETIMEDOUT or ecode == "timed out":
                        self.debug("test_poll_for_port_status: Connect "+str(ip)+":" +str(port)+ " timed out retrying. Time remaining("+str(timeout-elapsed)+")")
                except Exception, e:
                    self.debug('test_poll_for_port_status:'+str(ip)+':'+str(port)+' FAILED after attempts:'+str(attempt)+', elapsed:'+str(elapsed)+', err:'+str(e) )
                time.sleep(interval)
                elapsed = int(time.time() -start)    
        raise Exception('test_poll_for_port_status:'+str(ip)+':'+str(port)+' FAILED after attempts:'+str(attempt)+', elapsed:'+str(elapsed)+' seconds')
            
    def get_windows_emi(self):
        emi = None
        if self.emi is not None:
            self.debug("get_windows_emi returning provided emi:"+str(self.emi))
            emi = self.emi
        elif self.emi_location is not None:
            self.debug("get_windows_emi returning emi based on emi location"+str(self.emi_location))
            emi = self.tester.get_emi(location=self.emi_location)
        elif self.image_path is not None:
            self.debug("get_windows_emi attempting to create emi from image at path:"+str(self.image_path))
            emi = self.create_windows_emi_from_file(self.image_path)
        elif self.url is not None:
            self.debug("get_windows_emi attempting to create emi from url:"+str(self.url))
            emi = self.create_windows_emi_from_url(self.url)
        if not emi:
            raise Exception('test_get_windows_emi failed to get emi')
        self.emi = emi
        return emi
    
    def find_windows_instance(self,emi=None, location=None):
        self.debug('find_windows_instance attempting to find a pre-existing and running instance...')
        emi = emi or self.emi
        if not emi and location is not None:
            emi = self.get_images_by_location(location)
        if not emi:
            raise Exception("find_windows_instance: No emi provided, and could not find emi to match against instances")
        self.debug("Using emi:"+str(emi)+" to look for running instances...")
        instances = self.tester.get_instances(state='running',image_id=emi.id )
        self.debug("Returning instance list:"+str(instances))
        return instances
    
    def get_windows_instance(self):
        self.debug("get_windows_instance, check to see if we've been provided a running instance...")
        if self.instance:
            self.instance.update()
            if self.instance.state == 'running':
                return
        instances = self.find_windows_instance()
        for instance in instances:
            if instance.state =='running':
                try:
                    keypair = self.get_local_key_for_instance(instance)
                    self.instance = instance
                    self.keypair = keypair
                    return instance
                except: pass
        #We need to create a new instance...
        self.get_windows_emi()
        self.test_run_windows_emi()
        
    def get_local_key_for_instance(self,instance,keypath=None, exten=".pem"):
        self.debug("Looking for local keys for instance:"+str(instance.id))
        keypath = keypath or self.instance_keypath
        keys = self.tester.get_all_current_local_keys(path=keypath, exten=exten)
        for key in keys:
            if key.name == instance.key_name:
                self.debug("Found key:"+str(key.name))
                return key
        raise Exception("No local key found for "+str(instance.id)+":"+str(instance.key_name)+", at path:"+str(keypath)+" exten:"+str(exten))
        
        
    def get_free_ebs_devname(self, instance=None, max=16):
        self.debug('get_free_ebs_dev_name starting...')
        instance = instance or self.instance
        attachedvols = self.tester.get_volumes(attached_instance=instance.id)
        count=0
        for x in xrange(0,(max-1)):
            d = chr(ord('c') + x)
            dev = '/dev/sd'+str(d)
            in_use = False
            for volume in attachedvols:
                if volume.attach_data.device == dev:
                    in_use = True
                    count += 1
                    continue
            if not in_use:
                self.debug("get_free_ebs_devname: Got "+str(dev))
                return dev
        raise Exception('Instance:'+str(instance.id)+", no free devs. Has "+str(count)+" devices in use by ebs already. max:"+str(max))    
        

           
    def test_attach_single_volume(self, instance=None, dev=None):
        instance = instance or self.instance
        if instance is None:
            raise Exception('test_attach_single_volume: instance is None')
        vol = None
        for zone in self.zonelist:
            self.debug('Checking for available volumes in zone:'+str(zone.name))
            if zone.name == instance.placement:
                self.debug('Checking Volume:'+str(zone.name))
                for volume in zone.volumes:
                    if volume.status == 'available':
                        self.debug('Found Available Volume:'+str(volume.id))
                        vol = volume
                        break
        if not vol:
            raise Exception('test_attach_single_volume, no available volumes')
        if dev is None:
            dev = self.get_free_ebs_devname(instance=instance)
        self.tester.attach_volume(self.instance, vol, device_path=dev)
        
    def test_proxy_ebs_guest_attachment(self, instance=None, volume=None, wait=60):
        self.debug('test_proxy_ebs_guest_attachment starting...')
        fail = None
        instance = instance or self.instance
        if not instance:
            raise Exception("test_proxy_ebs_guest_attachment, instance is None")
        if volume:
            volume.update()
            if volume.attach_data.instance_id != instance.id:
                raise Exception('Volume:'+str(volume.id)+" not attached to:"+str(instance.id) )
            vols = [volume]
        else:
            vols = self.tester.get_volumes(attached_instance=instance.id)
        for volume in vols:
            #Make sure some time has passed before we test on the guest side before running guest test...
            attached_seconds = self.tester.get_volume_time_attached(volume)
            sleeptime =  0 if attached_seconds > wait else (wait - attached_seconds)
            self.debug("Volume has been attached for "+str(attached_seconds)+" seconds, waiting:"+str(sleeptime)+" for guest to detect attached vol")
            time.sleep(sleeptime)
            self.debug("Running Proxy ebs test now...")
            try:
                self.proxy.ps_ebs_test(retryinterval=30)
            except Exception, e:
                fail = e+"test_proxy_ebs_guest_attachment, "+str(instance)+"/"+str(volume)+", failed:"+str(e)+"\n"
        if fail:
            raise Exception(fail)
                
        
    def is_kvm(self, component=None):
        component = component or self.component or self.tester.get_component_machines("nc")[0]
        if (component.distro.name == machine.DistroName.rhel or component.distro.name == machine.DistroName.rhel) and int(component.distro.distro_number) < 6:
            return False
        else:
            return True
        
    def setup_active_dir_dns(self,zone=None,ad_dns=None):
        ad_dns = ad_dns or self.win_proxy_hostname 
        if not ad_dns:
            if self.proxy:
                ad_dns = self.proxy.proxy_hostname
            else:
                raise Exception('Need hostname/ip of AD DNS to use?')
        if not zone:
            for zone in self.zonelist:
                if zone.name == self.instance.placement:
                    break
        if not zone:
            raise Exception('setup_active_dir_dns zone unknown')
        self.debug("setup_active_dir_dns starting, zone:"+str(zone)+", dns:"+str(ad_dns))
        ccs = zone.partition.ccs
        for cc in ccs:
            #update the CC of this zone to use this DNS server...
            cmd = 'sed -i \'s/VNET_DNS.*$/VNET_DNS="'+str(ad_dns)+'"/g\' '+str(self.tester.eucapath)+'/etc/eucalyptus/eucalyptus.conf'
            self.debug('Attempting to update VNET DNS on:'+str(cc.hostname)+', cmd:'+str(cmd))
            cc.machine.sys(cmd,code=0)
            cc.stop()    
        zone.partition.service_manager.wait_for_service(cc, state='NOTREADY',timeout=120)
        for cc in ccs:
            cc.start()
        zone.partition.service_manager.wait_for_service(cc, state='ENABLED',timeout=120)
        self.debug("setup_active_dir VNET_DNS done")
    
    
    def basic_proxy_test_suite(self, instance=None):
        instance = instance or self.instance
        list = []
        if instance is None or instance.state != 'running':
            self.debug("basic_proxy_test_suite: No running instances found, creating instance now")
            test = self.create_testcase_from_method(self.get_windows_emi)
            test.eof = True
            list.append(test)
            test = self.create_testcase_from_method(self.get_windows_instance)
            test.eof = True
            list.append(test)
        list.append(self.create_testcase_from_method(self.test_get_windows_instance_password))
        test = self.create_testcase_from_method(self.test_poll_for_rdp_port_status)
        test.eof=True
        list.append(test)
        test = self.create_testcase_from_method(self.update_proxy_instance_data)
        test.eof=True
        list.append(test)
        list.append(self.create_testcase_from_method(self.test_wait_for_instance_boot))
        test = self.create_testcase_from_method(self.proxy.ps_login_test)
        test.eof=True
        list.append(test)
        list.append(self.create_testcase_from_method(self.proxy.ps_ephemeral_test))
        list.append(self.create_testcase_from_method(self.proxy.ps_hostname_test))
        list.append(self.create_testcase_from_method(self.setup_windows_test_volumes))
        list.append(self.create_testcase_from_method(self.proxy.ps_hostname_test))
        list.append(self.create_testcase_from_method(self.test_attach_single_volume))
        #The guest test may not work on a 32bit host...
        list.append(self.create_testcase_from_method(self.test_proxy_ebs_guest_attachment))
        if self.is_kvm(): 
            list.append(self.create_testcase_from_method(self.proxy.ps_virtio_test))
        else:
            list.append(self.create_testcase_from_method(self.proxy.ps_xenpv_test))
        test = self.create_testcase_from_method(self.setup_active_dir_dns)
        test.eof = True
        list.append(test)
        list.append(self.create_testcase_from_method(self.proxy.ps_admembership_test))
        list.append(self.create_testcase_from_method(self.proxy.ps_eucaadkey_test))
        list.append(self.create_testcase_from_method(self.proxy.ps_rdpermission_test))
        #Run this test case list only exit on fail if a given test method has the flag set. 
        self.run_test_case_list(list, eof=False)



    def simple_windows_test_suite(self,instance=None, clean_on_exit=False):
        """
        Runs a basic subset of windows tests against either a running instance, or will try to find
        a Windows emi by location string and create a running instance to test from that emi.
        :param instance: boto instance object to run windows tests against.

        """
        instance = instance or self.instance
        tests = []
        if instance is None or instance.state != 'running':
            self.debug("basic_proxy_test_suite: No running instances found, creating instance now")
            test = self.create_testcase_from_method(self.get_windows_emi)
            test.eof = True
            tests.append(test)
            test = self.create_testcase_from_method(self.get_windows_instance)
            test.eof = True
            tests.append(test)
        tests.append(self.create_testunit_from_method(self.test_get_windows_instance_password, eof=True))
        tests.append(self.create_testunit_from_method(self.update_proxy_info, eof=True))
        tests.append(self.create_testunit_from_method(self.test_wait_for_instance_boot))
        tests.append(self.create_testunit_from_method(self.test_poll_for_rdp_port_status, eof=True))
        tests.append(self.create_testunit_from_method(self.proxy.ps_login_test, eof=True))
        tests.append(self.create_testunit_from_method(self.proxy.ps_ephemeral_test))
        tests.append(self.create_testunit_from_method(self.proxy.ps_hostname_test))
        tests.append(self.create_testunit_from_method(self.proxy.ps_virtio_test))
        self.run_test_case_list(tests, clean_on_exit=clean_on_exit, eof=False)

    #def windows_bundle_test_from_emi(self, emi, timeout_minutes=35):




    def windows_bundle_test_from_instance(self, instance, timeout_minutes=25):
        timeout = 60 * timeout_minutes
        cli = self.tester
    
        
    
        
        
        
        



# Copyright 2011-2012 Eucalyptus Systems, Inc.
#
# Redistribution and use of this software in source and binary forms,
# with or without modification, are permitted provided that the following
# conditions are met:
#
#   Redistributions of source code must retain the above copyright notice,
#   this list of conditions and the following disclaimer.
#
#   Redistributions in binary form must reproduce the above copyright
#   notice, this list of conditions and the following disclaimer in the
#   documentation and/or other materials provided with the distribution.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

from eutester import Eutester
import boto
from boto.ec2.regioninfo import RegionInfo

EC2RegionData = {
    'us-east-1' : 'ec2.us-east-1.amazonaws.com',
    'us-west-1' : 'ec2.us-west-1.amazonaws.com',
    'eu-west-1' : 'ec2.eu-west-1.amazonaws.com',
    'ap-northeast-1' : 'ec2.ap-northeast-1.amazonaws.com',
    'ap-southeast-1' : 'ec2.ap-southeast-1.amazonaws.com'}

class STSops(Eutester):

    def __init__(self, endpoint=None, region=None, credpath=None, aws_access_key_id=None, aws_secret_access_key=None):
        self.aws_access_key_id = aws_access_key_id
        self.aws_secret_access_key = aws_secret_access_key
        self.user_id = None
        self.account_id = None
        super(STSops, self).__init__(credpath=credpath)
        self.setup_sts_connection(endpoint=endpoint, region=region, aws_access_key_id=self.aws_access_key_id, aws_secret_access_key=self.aws_secret_access_key)

    def setup_sts_connection(self, endpoint=None, region=None, aws_access_key_id=None, aws_secret_access_key=None, path="/",port=443, is_secure=True, boto_debug=0):
        sts_region = RegionInfo()
        if region:
            self.debug("Check region: " + str(region))
            try:
                if not endpoint:
                    sts_region.endpoint = EC2RegionData[region]
                else:
                    sts_region.endpoint = endpoint
            except KeyError:
                raise Exception( 'Unknown region: %s' % region)
        else:
            sts_region.name = 'eucalyptus'
            if endpoint:
                sts_region.endpoint = endpoint
            else:
                sts_region.endpoint = self.get_ec2_ip()

        try:
            sts_connection_args = { 'aws_access_key_id' : aws_access_key_id,
                                    'aws_secret_access_key': aws_secret_access_key,
                                    'is_secure': is_secure,
                                    'debug':boto_debug,
                                    'port' : port,
                                    'path' : path,
                                    'region' : sts_region}
            self.debug("Attempting to create STS connection to " + self.get_ec2_ip() + ':' + str(port) + path)
            self.tokens = boto.connect_sts(**sts_connection_args)
        except Exception, e:
            self.critical("Was unable to create STS connection because of exception: " + str(e))

    def get_session_token( self, duration=None ):
        '''
        Get a possibly cached session token, if getting a new token request the given duration
        Options:
            duration - The desired duration for the token in seconds (if issued, None for default duration)
        '''
        return self.tokens.get_session_token( duration )

    def issue_session_token( self, duration=None ):
        '''
        Get a newly issued session token with the given (or default) duration
        Options:
            duration - The desired duration for the token in seconds (None for default duration)
        '''
        return self.tokens.get_session_token( duration, force_new=True )

